!
!! This subroutine is the interface between Fortran 77 main program
!! and SECHIBA.
!! - Input fields are gathered to keep just continental points
!! - call sechiba_main That's SECHIBA process.
!! - Output fields are scattered to complete global fields
!!
!! @call sechiba_main
!! @Version : $Revision: 1.36.2.5 $, $Date: 2004/04/01 10:02:56 $
!!
!! @author Marie-Alice Foujols and Jan Polcher
!! 
!! $Header: /home/ssipsl/CVSREP/ORCHIDEE/src_sechiba/intersurf.f90,v 1.36.2.5 2004/04/01 10:02:56 ssipsl Exp $
!!
!f90doc MODULEintersurf
MODULE intersurf

  USE IOIPSL

  USE reqdprec
  USE sechiba
  USE constantes
  USE constantes_veg

  IMPLICIT NONE

  PRIVATE
  PUBLIC :: intersurf_main, init_intersurf

  INTERFACE intersurf_main
    MODULE PROCEDURE intersurf_main_2d, intersurf_main_1d, intersurf_gathered
  END INTERFACE
  !
  !  Global variables
  !
  INTEGER(long),PARAMETER                           :: max_hist_level = 10
  !
  LOGICAL, SAVE                                     :: l_first_intersurf=.TRUE. !! Initialisation has to be done one time
  !
  INTEGER(long), SAVE                               :: hist_id, rest_id        !! IDs for history and restart files
  INTEGER(long), SAVE                               :: hist_id_stom, rest_id_stom !! Dito for STOMATE
  !
  INTEGER(long), SAVE                               :: itau_offset  !! This offset is used to phase the 
  !                                                                 !! calendar of the GCM or the driver.
  REAL(stnd)                                        :: date0_shifted
  !
  TYPE(control_type), SAVE                          :: control_flags !! Flags that (de)activate parts of the model
  !
  REAL(stnd), ALLOCATABLE, DIMENSION (:,:), SAVE    :: lalo          !! Geographical coordinates
  REAL(stnd), ALLOCATABLE, DIMENSION (:), SAVE      :: contfrac      !! Fraction of continents. Computed once
  INTEGER(long), ALLOCATABLE, DIMENSION (:,:), SAVE :: neighbours    !! neighoring grid points if land
  REAL(stnd), ALLOCATABLE, DIMENSION (:,:), SAVE    :: resolution    !! size in x an y of the grid (m)
  !
  !
  ! At module level we need the ids of the variables for the ORCHIDEE_WATCH. They will be 
  ! shared by the intersurf_initwatch and intersurf_wrtwatch routines.
  ! The flag which will control all this is watchout
  !
  LOGICAL,SAVE            :: watchout = .FALSE.
  INTEGER(long),SAVE      :: time_id, timestp_id
  INTEGER(long),SAVE      :: watchfid, soldownid, rainfid, snowfid, lwradid, psolid, tairid, qairid, uid, vid
  INTEGER(long),SAVE      :: watchoffset
  CHARACTER(LEN=80),SAVE  :: watchout_file
  !
CONTAINS
  !
  !f90doc CONTAINS
  ! 
  SUBROUTINE intersurf_main_2d (kjit, iim, jjm, kjpindex, kindex, xrdt, &
     & lrestart_read, lrestart_write, lon, lat, zcontfrac, date0, &
! First level conditions
     & zlev, u, v, qair, temp_air, epot_air, ccanopy, &
! Variables for the implicit coupling
     & cdrag, petAcoef, peqAcoef, petBcoef, peqBcoef, &
! Rain, snow, radiation and surface pressure
     & precip_rain, precip_snow, lwdown, swnet, swdown, pb, &
! Output : Fluxes
     & vevapp, fluxsens, fluxlat, coastalflow, riverflow, &
! Surface temperatures and surface properties
     & tsol_rad, temp_sol_new, qsurf, albedo, emis, z0) 
    
    ! routines called : sechiba_main
    !
    IMPLICIT NONE
    !    
    ! interface description for dummy arguments
    ! input scalar 
    INTEGER(long),INTENT (in)                            :: kjit          !! Time step number
    INTEGER(long),INTENT (in)                            :: iim, jjm      !! Dimension of input fields
    INTEGER(long),INTENT (in)                            :: kjpindex      !! Number of continental points
    REAL(stnd),INTENT (in)                               :: xrdt          !! Time step in seconds
    LOGICAL, INTENT (in)                                 :: lrestart_read !! Logical for _restart_ file to read
    LOGICAL, INTENT (in)                                 :: lrestart_write!! Logical for _restart_ file to write'
    REAL(stnd), INTENT (in)                              :: date0         !! Date at which kjit = 0
    ! input fields
    INTEGER(long),DIMENSION (kjpindex), INTENT (in)      :: kindex        !! Index for continental points
    REAL(stnd),DIMENSION (iim,jjm), INTENT(in)             :: u             !! Lowest level wind speed
    REAL(stnd),DIMENSION (iim,jjm), INTENT(in)             :: v             !! Lowest level wind speed 
    REAL(stnd),DIMENSION (iim,jjm), INTENT(in)             :: zlev          !! Height of first layer
    REAL(stnd),DIMENSION (iim,jjm), INTENT(in)             :: qair          !! Lowest level specific humidity
    REAL(stnd),DIMENSION (iim,jjm), INTENT(in)             :: precip_rain   !! Rain precipitation
    REAL(stnd),DIMENSION (iim,jjm), INTENT(in)             :: precip_snow   !! Snow precipitation
    REAL(stnd),DIMENSION (iim,jjm), INTENT(in)             :: lwdown        !! Down-welling long-wave flux 
    REAL(stnd),DIMENSION (iim,jjm), INTENT(in)             :: swnet         !! Net surface short-wave flux
    REAL(stnd),DIMENSION (iim,jjm), INTENT(in)             :: swdown        !! Downwelling surface short-wave flux
    REAL(stnd),DIMENSION (iim,jjm), INTENT(in)             :: temp_air      !! Air temperature in Kelvin
    REAL(stnd),DIMENSION (iim,jjm), INTENT(in)             :: epot_air      !! Air humidity
    REAL(stnd),DIMENSION (iim,jjm), INTENT(in)             :: ccanopy       !! CO2 concentration in the canopy
    REAL(stnd),DIMENSION (iim,jjm), INTENT(in)             :: petAcoef      !! Coeficients A from the PBL resolution
    REAL(stnd),DIMENSION (iim,jjm), INTENT(in)             :: peqAcoef      !! One for T and another for q
    REAL(stnd),DIMENSION (iim,jjm), INTENT(in)             :: petBcoef      !! Coeficients B from the PBL resolution
    REAL(stnd),DIMENSION (iim,jjm), INTENT(in)             :: peqBcoef      !! One for T and another for q
    REAL(stnd),DIMENSION (iim,jjm), INTENT(inout)          :: cdrag         !! Cdrag
    REAL(stnd),DIMENSION (iim,jjm), INTENT(in)             :: pb            !! Lowest level pressure
    REAL(stnd),DIMENSION (iim,jjm), INTENT(in)             :: lon, lat      !! Geographical coordinates
    REAL(stnd),DIMENSION (iim,jjm), INTENT(in)             :: zcontfrac      !! Fraction of continent in the grid
    ! output fields
    REAL(stnd),DIMENSION (iim,jjm), INTENT(out)            :: z0            !! Surface roughness
    REAL(stnd),DIMENSION (iim,jjm), INTENT(out)            :: coastalflow   !! Diffuse flow of water into the ocean (m^3/dt)
    REAL(stnd),DIMENSION (iim,jjm), INTENT(out)            :: riverflow     !! Largest rivers flowing into the ocean (m^3/dt)
    REAL(stnd),DIMENSION (iim,jjm), INTENT(out)            :: tsol_rad      !!
    REAL(stnd),DIMENSION (iim,jjm), INTENT(out)            :: vevapp        !! Total of evaporation
    REAL(stnd),DIMENSION (iim,jjm), INTENT(out)            :: temp_sol_new  !! New soil temperature
    REAL(stnd),DIMENSION (iim,jjm), INTENT(out)            :: qsurf         !! Surface specific humidity
    REAL(stnd),DIMENSION (iim,jjm,2), INTENT(out)          :: albedo        !! Albedo
    REAL(stnd),DIMENSION (iim,jjm), INTENT(out)            :: fluxsens      !! Sensible chaleur flux
    REAL(stnd),DIMENSION (iim,jjm), INTENT(out)            :: fluxlat       !! Latent chaleur flux
    REAL(stnd),DIMENSION (iim,jjm), INTENT(out)            :: emis          !! Emissivity
    ! LOCAL declaration
    ! work arrays to scatter and/or gather information just before/after sechiba_main call's
    ! and to keep output value for next call
    REAL(stnd),DIMENSION (kjpindex)                      :: zu            !! Work array to keep u
    REAL(stnd),DIMENSION (kjpindex)                      :: zv            !! Work array to keep v
    REAL(stnd),DIMENSION (kjpindex)                      :: zzlev         !! Work array to keep zlev
    REAL(stnd),DIMENSION (kjpindex)                      :: zqair         !! Work array to keep qair
    REAL(stnd),DIMENSION (kjpindex)                      :: zprecip_rain  !! Work array to keep precip_rain
    REAL(stnd),DIMENSION (kjpindex)                      :: zprecip_snow  !! Work array to keep precip_snow
    REAL(stnd),DIMENSION (kjpindex)                      :: zlwdown       !! Work array to keep lwdown
    REAL(stnd),DIMENSION (kjpindex)                      :: zswnet        !! Work array to keep swnet
    REAL(stnd),DIMENSION (kjpindex)                      :: zswdown       !! Work array to keep swdown
    REAL(stnd),DIMENSION (kjpindex)                      :: ztemp_air     !! Work array to keep temp_air
    REAL(stnd),DIMENSION (kjpindex)                      :: zepot_air     !! Work array to keep epot_air
    REAL(stnd),DIMENSION (kjpindex)                      :: zccanopy      !! Work array to keep ccanopy
    REAL(stnd),DIMENSION (kjpindex)                      :: zpetAcoef     !! Work array to keep petAcoef
    REAL(stnd),DIMENSION (kjpindex)                      :: zpeqAcoef     !! Work array to keep peqAcoef
    REAL(stnd),DIMENSION (kjpindex)                      :: zpetBcoef     !! Work array to keep petBcoef
    REAL(stnd),DIMENSION (kjpindex)                      :: zpeqBcoef     !! Work array to keep peqVcoef
    REAL(stnd),DIMENSION (kjpindex)                      :: zcdrag        !! Work array to keep cdrag
    REAL(stnd),DIMENSION (kjpindex)                      :: zpb           !! Work array to keep pb
    REAL(stnd),DIMENSION (kjpindex)                      :: zz0           !! Work array to keep z0
    REAL(stnd),DIMENSION (kjpindex)                      :: zcoastal      !! Work array to keep coastalflow
    REAL(stnd),DIMENSION (kjpindex)                      :: zriver        !! Work array to keep riverflow
    REAL(stnd),DIMENSION (kjpindex)                      :: dcoastal      !! Work array to keep coastalflow
    REAL(stnd),DIMENSION (kjpindex)                      :: driver        !! Work array to keep riverflow
    REAL(stnd),DIMENSION (kjpindex)                      :: ztsol_rad     !! Work array to keep tsol_rad
    REAL(stnd),DIMENSION (kjpindex)                      :: zvevapp       !! Work array to keep vevapp
    REAL(stnd),DIMENSION (kjpindex)                      :: ztemp_sol_new !! Work array to keep temp_sol_new
    REAL(stnd),DIMENSION (kjpindex)                      :: zqsurf        !! Work array to keep qsurf
    REAL(stnd),DIMENSION (kjpindex,2)                    :: zalbedo       !! Work array to keep albedo
    REAL(stnd),DIMENSION (kjpindex)                      :: zfluxsens     !! Work array to keep fluxsens
    REAL(stnd),DIMENSION (kjpindex)                      :: zfluxlat      !! Work array to keep fluxlat
    REAL(stnd),DIMENSION (kjpindex)                      :: zemis         !! Work array to keep emis
    !
    ! Local variables with shape of the inputs
    !
    REAL(stnd),DIMENSION (iim,jjm)                       :: dswnet         !! Net surface short-wave flux
    !
    INTEGER(long)                                        :: i, j, ik
    INTEGER(long)                                        :: itau_sechiba
    REAL(stnd)                                           :: zlev_mean
    LOGICAL :: check = .FALSE.

    IF (l_first_intersurf) THEN
       !
       IF ( check ) WRITE(*,*) 'Initialisation of intersurf'
       !
       !  Configuration of SSL specific parameters
       !
       CALL intsurf_config(control_flags)
       !
       !  Create the internal coordinate table
       !
       IF ( (.NOT.ALLOCATED(lalo))) THEN
          ALLOCATE(lalo(kjpindex,2))
       ENDIF
       !
       DO ik=1,kjpindex
          
          j = ((kindex(ik)-1)/iim) + 1
          i = (kindex(ik) - (j-1)*iim)

          lalo(ik,1) = lat(i,j)
          lalo(ik,2) = lon(i,j)

       ENDDO
       !-
       !- Compute variable to help describe the grid
       !- once the points are gathered.
       !-
       IF ( (.NOT.ALLOCATED(neighbours))) THEN
          ALLOCATE(neighbours(kjpindex,8))
       ENDIF
       IF ( (.NOT.ALLOCATED(resolution))) THEN
          ALLOCATE(resolution(kjpindex,2))
       ENDIF
       !
       CALL grid_stuff (kjpindex, iim, jjm, lon, lat, kindex, neighbours, resolution)
       !
       !- Store the fraction of the continents only once so that the user
       !- does not change them afterwards.
       !
       IF ( (.NOT.ALLOCATED(contfrac))) THEN
          ALLOCATE(contfrac(kjpindex))
       ENDIF
       !
       DO ik=1,kjpindex
          
          j = ((kindex(ik)-1)/iim) + 1
          i = (kindex(ik) - (j-1)*iim)
          
          contfrac(ik) = zcontfrac(i,j)

       ENDDO
       !
       CALL intsurf_restart(kjit, iim, jjm, lon, lat, date0, xrdt, control_flags, rest_id, rest_id_stom, itau_offset)
       !
       CALL intsurf_history(iim, jjm, lon, lat, kjit+itau_offset, date0_shifted, xrdt, control_flags, hist_id, hist_id_stom)
       !
       IF ( watchout ) THEN

          zlev_mean = SUM(zlev)/REAL(iim*jjm,stnd)

          CALL intersurf_initwatch(iim, jjm, kjpindex, date0, kjit+itau_offset, xrdt, kindex, lon, lat, zlev_mean, contfrac)

       ENDIF
       !
       IF ( check ) WRITE(*,*) 'End of Initialisation of intersurf'
       !
    ENDIF

    !
    ! 1. gather input fields from kindex array
    !    Warning : I'm not sure this interface with one dimension array is the good one

    DO ik=1, kjpindex
      
       j = ((kindex(ik)-1)/iim) + 1
       i = (kindex(ik) - (j-1)*iim)
       
       zu(ik)           = u(i,j)
       zv(ik)           = v(i,j)
       zzlev(ik)        = zlev(i,j)
       zqair(ik)        = qair(i,j)
       zprecip_rain(ik) = precip_rain(i,j)*xrdt
       zprecip_snow(ik) = precip_snow(i,j)*xrdt
       zlwdown(ik)      = lwdown(i,j)
       zswnet(ik)       = swnet(i,j)
       zswdown(ik)      = swdown(i,j)
       ztemp_air(ik)    = temp_air(i,j)
       zepot_air(ik)    = epot_air(i,j)
       zccanopy(ik)     = ccanopy(i,j)
       zpetAcoef(ik)    = petAcoef(i,j)
       zpeqAcoef(ik)    = peqAcoef(i,j)
       zpetBcoef(ik)    = petBcoef(i,j)
       zpeqBcoef(ik)    = peqBcoef(i,j)
       zcdrag(ik)       = cdrag(i,j)
       zpb(ik)          = pb(i,j)
       
    ENDDO
    
    !
    ! 3. call sechiba for continental points only
    !
    IF ( check ) WRITE(*,*) 'Calling sechiba'
    !
    !  Shift the time step to phase the two models
    !
    itau_sechiba = kjit + itau_offset
    !
    IF ( watchout ) THEN

       IF ( .NOT. l_first_intersurf) THEN
          CALL intersurf_wrtwatch(kjpindex, itau_sechiba, xrdt, zswdown, zprecip_rain, &
               &   zprecip_snow, zlwdown, zpb, ztemp_air, zqair, zu, zv)
       ENDIF

    ENDIF
    !
    CALL sechiba_main (itau_sechiba, iim*jjm, kjpindex, kindex, xrdt, date0_shifted, &
       & lrestart_read, lrestart_write, control_flags, &
       & lalo, contfrac, neighbours, resolution, &
! First level conditions
       & zzlev, zu, zv, zqair, ztemp_air, zepot_air, zccanopy, &
! Variables for the implicit coupling
       & zcdrag, zpetAcoef, zpeqAcoef, zpetBcoef, zpeqBcoef, &
! Rain, snow, radiation and surface pressure
       & zprecip_rain ,zprecip_snow,  zlwdown, zswnet, zswdown, zpb, &
! Output : Fluxes
       & zvevapp, zfluxsens, zfluxlat, zcoastal, zriver, &
! Surface temperatures and surface properties
       & ztsol_rad, ztemp_sol_new, zqsurf, zalbedo, zemis, zz0, &
! File ids
       & rest_id, hist_id, rest_id_stom, hist_id_stom ) 
    
    !
    IF ( check ) WRITE(*,*) 'out of SECHIBA'
    !
    ! 4. scatter output fields
    !
    z0(:,:)           = undef_sechiba
    coastalflow(:,:)  = undef_sechiba
    riverflow(:,:)    = undef_sechiba
    tsol_rad(:,:)     = undef_sechiba
    vevapp(:,:)       = undef_sechiba
    temp_sol_new(:,:) = undef_sechiba 
    qsurf(:,:)        = undef_sechiba 
    albedo(:,:,:)     = undef_sechiba
    albedo(:,:,:)     = undef_sechiba
    fluxsens(:,:)     = undef_sechiba
    fluxlat(:,:)      = undef_sechiba
    emis(:,:)         = undef_sechiba 
    cdrag(:,:)        = undef_sechiba 
    dswnet(:,:)       = undef_sechiba 
    !
    DO ik=1, kjpindex
      
    
       j = ((kindex(ik)-1)/iim) + 1
       i = (kindex(ik) - (j-1)*iim)

       z0(i,j)           = zz0(ik)
       coastalflow(i,j)  = zcoastal(ik)/1000.
       riverflow(i,j)    = zriver(ik)/1000.
       tsol_rad(i,j)     = ztsol_rad(ik)
       vevapp(i,j)       = zvevapp(ik)
       temp_sol_new(i,j) = ztemp_sol_new(ik)
       qsurf(i,j)        = zqsurf(ik)
       albedo(i,j,1)     = zalbedo(ik,1)
       albedo(i,j,2)     = zalbedo(ik,2)
       fluxsens(i,j)     = zfluxsens(ik)
       fluxlat(i,j)      = zfluxlat(ik)
       emis(i,j)         = zemis(ik)
       cdrag(i,j)        = zcdrag(ik)
       dswnet(i,j)       = zswnet(ik)

    ENDDO
    !
    ! Modified fields for variables scattered during the writing
    !
    dcoastal(:) = (zcoastal(:))/1000.     
    driver(:)   = (zriver(:))/1000.
    !
    IF ( .NOT. l_first_intersurf) THEN
       !
       IF ( .NOT. almaoutput ) THEN
       !
       !  scattered during the writing
       !
          CALL histwrite (hist_id, 'evap',     itau_sechiba, zvevapp, kjpindex, kindex)
          CALL histwrite (hist_id, 'coastalflow',itau_sechiba, dcoastal, kjpindex, kindex)
          CALL histwrite (hist_id, 'riverflow',itau_sechiba, driver, kjpindex, kindex)
       ! 
          CALL histwrite (hist_id, 'temp_sol', itau_sechiba, temp_sol_NEW, iim*jjm, kindex)
          CALL histwrite (hist_id, 'tsol_max', itau_sechiba, temp_sol_NEW, iim*jjm, kindex)
          CALL histwrite (hist_id, 'tsol_min', itau_sechiba, temp_sol_NEW, iim*jjm, kindex)
          CALL histwrite (hist_id, 'fluxsens', itau_sechiba, fluxsens, iim*jjm, kindex)
          CALL histwrite (hist_id, 'fluxlat',  itau_sechiba, fluxlat, iim*jjm, kindex)
          CALL histwrite (hist_id, 'swnet',    itau_sechiba, dswnet, iim*jjm, kindex)
          CALL histwrite (hist_id, 'alb_vis',  itau_sechiba, albedo(:,:,1), iim*jjm, kindex)
          CALL histwrite (hist_id, 'alb_nir',  itau_sechiba, albedo(:,:,2), iim*jjm, kindex)
          CALL histwrite (hist_id, 'tair',     itau_sechiba, temp_air, iim*jjm, kindex)
          CALL histwrite (hist_id, 'qair',     itau_sechiba, qair, iim*jjm, kindex)
       ELSE
          CALL histwrite (hist_id, 'Evap', itau_sechiba, zvevapp, kjpindex, kindex)
          CALL histwrite (hist_id, 'SWnet',    itau_sechiba, dswnet, iim*jjm, kindex)
          CALL histwrite (hist_id, 'Qh', itau_sechiba, fluxsens, iim*jjm, kindex)
          CALL histwrite (hist_id, 'Qle',  itau_sechiba, fluxlat, iim*jjm, kindex)
          CALL histwrite (hist_id, 'AvgSurfT', itau_sechiba, temp_sol_NEW, iim*jjm, kindex)
          CALL histwrite (hist_id, 'RadT', itau_sechiba, temp_sol_NEW, iim*jjm, kindex)
       ENDIF
       !
    ENDIF
    !
    !
    ! 5.0 Transform the water fluxes into Kg/m^2s and m^3/s
    !
    DO ik=1, kjpindex
    
       j = ((kindex(ik)-1)/iim) + 1
       i = (kindex(ik) - (j-1)*iim)

       vevapp(i,j) = vevapp(i,j)/xrdt
       coastalflow(i,j) = coastalflow(i,j)/xrdt
       riverflow(i,j) = riverflow(i,j)/xrdt

    ENDDO
    !
    IF ( lrestart_write .AND. watchout ) THEN

       CALL intersurf_clowatch()

    ENDIF
    !
    l_first_intersurf = .FALSE.
    !
    IF (long_print) WRITE (numout,*) ' intersurf_main done '
    
  END SUBROUTINE intersurf_main_2d
!
  SUBROUTINE intersurf_main_1d (kjit, iim, jjm, kjpindex, kindex, xrdt, &
     & lrestart_read, lrestart_write, lon, lat, zcontfrac, date0, &
! First level conditions
     & zlev,  u, v, qair, temp_air, epot_air, ccanopy, &
! Variables for the implicit coupling
     & cdrag, petAcoef, peqAcoef, petBcoef, peqBcoef, &
! Rain, snow, radiation and surface pressure
     & precip_rain, precip_snow, lwdown, swnet, swdown, pb, &
! Output : Fluxes
     & vevapp, fluxsens, fluxlat, coastalflow, riverflow, &
! Surface temperatures and surface properties
     & tsol_rad, temp_sol_new, qsurf, albedo, emis, z0) 
    
    ! routines called : sechiba_main
    !
    IMPLICIT NONE
    !    
    ! interface description for dummy arguments
    ! input scalar 
    INTEGER(long),INTENT (in)                            :: kjit          !! Time step number
    INTEGER(long),INTENT (in)                            :: iim, jjm      !! Dimension of input fields
    INTEGER(long),INTENT (in)                            :: kjpindex      !! Number of continental points
    REAL(stnd),INTENT (in)                               :: xrdt          !! Time step in seconds
    LOGICAL, INTENT (in)                                 :: lrestart_read !! Logical for _restart_ file to read
    LOGICAL, INTENT (in)                                 :: lrestart_write!! Logical for _restart_ file to write'
    REAL(stnd), INTENT (in)                              :: date0         !! Date at which kjit = 0
    ! input fields
    INTEGER(long),DIMENSION (kjpindex), INTENT (in)      :: kindex        !! Index for continental points
    REAL(stnd),DIMENSION (iim*jjm), INTENT(in)             :: u             !! Lowest level wind speed
    REAL(stnd),DIMENSION (iim*jjm), INTENT(in)             :: v             !! Lowest level wind speed 
    REAL(stnd),DIMENSION (iim*jjm), INTENT(in)             :: zlev          !! Height of first layer
    REAL(stnd),DIMENSION (iim*jjm), INTENT(in)             :: qair          !! Lowest level specific humidity
    REAL(stnd),DIMENSION (iim*jjm), INTENT(in)             :: precip_rain   !! Rain precipitation
    REAL(stnd),DIMENSION (iim*jjm), INTENT(in)             :: precip_snow   !! Snow precipitation
    REAL(stnd),DIMENSION (iim*jjm), INTENT(in)             :: lwdown        !! Down-welling long-wave flux 
    REAL(stnd),DIMENSION (iim*jjm), INTENT(in)             :: swnet         !! Net surface short-wave flux
    REAL(stnd),DIMENSION (iim*jjm), INTENT(in)             :: swdown        !! Downwelling surface short-wave flux
    REAL(stnd),DIMENSION (iim*jjm), INTENT(in)             :: temp_air      !! Air temperature in Kelvin
    REAL(stnd),DIMENSION (iim*jjm), INTENT(in)             :: epot_air      !! Air humidity
    REAL(stnd),DIMENSION (iim*jjm), INTENT(in)             :: ccanopy       !! CO2 concentration in the canopy
    REAL(stnd),DIMENSION (iim*jjm), INTENT(in)             :: petAcoef      !! Coeficients A from the PBL resolution
    REAL(stnd),DIMENSION (iim*jjm), INTENT(in)             :: peqAcoef      !! One for T and another for q
    REAL(stnd),DIMENSION (iim*jjm), INTENT(in)             :: petBcoef      !! Coeficients B from the PBL resolution
    REAL(stnd),DIMENSION (iim*jjm), INTENT(in)             :: peqBcoef      !! One for T and another for q
    REAL(stnd),DIMENSION (iim*jjm), INTENT(inout)          :: cdrag         !! Cdrag
    REAL(stnd),DIMENSION (iim*jjm), INTENT(in)             :: pb            !! Lowest level pressure
    REAL(stnd),DIMENSION (iim*jjm), INTENT(in)             :: lon, lat      !! Geographical coordinates
    REAL(stnd),DIMENSION (iim*jjm), INTENT(in)             :: zcontfrac     !! Fraction of continent
    ! output fields
    REAL(stnd),DIMENSION (iim*jjm), INTENT(out)            :: z0            !! Surface roughness
    REAL(stnd),DIMENSION (iim*jjm), INTENT(out)            :: coastalflow   !! Diffuse flow of water into the ocean (m^3/dt)
    REAL(stnd),DIMENSION (iim*jjm), INTENT(out)            :: riverflow     !! Largest rivers flowing into the ocean (m^3/dt)
    REAL(stnd),DIMENSION (iim*jjm), INTENT(out)            :: tsol_rad      !!
    REAL(stnd),DIMENSION (iim*jjm), INTENT(out)            :: vevapp        !! Total of evaporation
    REAL(stnd),DIMENSION (iim*jjm), INTENT(out)            :: temp_sol_new  !! New soil temperature
    REAL(stnd),DIMENSION (iim*jjm), INTENT(out)            :: qsurf         !! Surface specific humidity
    REAL(stnd),DIMENSION (iim*jjm,2), INTENT(out)          :: albedo        !! Albedo
    REAL(stnd),DIMENSION (iim*jjm), INTENT(out)            :: fluxsens      !! Sensible chaleur flux
    REAL(stnd),DIMENSION (iim*jjm), INTENT(out)            :: fluxlat       !! Latent chaleur flux
    REAL(stnd),DIMENSION (iim*jjm), INTENT(out)            :: emis          !! Emissivity
    ! LOCAL declaration
    ! work arrays to scatter and/or gather information just before/after sechiba_main call's
    ! and to keep output value for next call
    REAL(stnd),DIMENSION (kjpindex)                      :: zu            !! Work array to keep u
    REAL(stnd),DIMENSION (kjpindex)                      :: zv            !! Work array to keep v
    REAL(stnd),DIMENSION (kjpindex)                      :: zzlev         !! Work array to keep zlev
    REAL(stnd),DIMENSION (kjpindex)                      :: zqair         !! Work array to keep qair
    REAL(stnd),DIMENSION (kjpindex)                      :: zprecip_rain  !! Work array to keep precip_rain
    REAL(stnd),DIMENSION (kjpindex)                      :: zprecip_snow  !! Work array to keep precip_snow
    REAL(stnd),DIMENSION (kjpindex)                      :: zlwdown       !! Work array to keep lwdown
    REAL(stnd),DIMENSION (kjpindex)                      :: zswnet        !! Work array to keep swnet
    REAL(stnd),DIMENSION (kjpindex)                      :: zswdown       !! Work array to keep swdown
    REAL(stnd),DIMENSION (kjpindex)                      :: ztemp_air     !! Work array to keep temp_air
    REAL(stnd),DIMENSION (kjpindex)                      :: zepot_air     !! Work array to keep epot_air
    REAL(stnd),DIMENSION (kjpindex)                      :: zccanopy      !! Work array to keep ccanopy
    REAL(stnd),DIMENSION (kjpindex)                      :: zpetAcoef     !! Work array to keep petAcoef
    REAL(stnd),DIMENSION (kjpindex)                      :: zpeqAcoef     !! Work array to keep peqAcoef
    REAL(stnd),DIMENSION (kjpindex)                      :: zpetBcoef     !! Work array to keep petBcoef
    REAL(stnd),DIMENSION (kjpindex)                      :: zpeqBcoef     !! Work array to keep peqVcoef
    REAL(stnd),DIMENSION (kjpindex)                      :: zcdrag        !! Work array to keep cdrag
    REAL(stnd),DIMENSION (kjpindex)                      :: zpb           !! Work array to keep pb
    REAL(stnd),DIMENSION (kjpindex)                      :: zz0           !! Work array to keep z0
    REAL(stnd),DIMENSION (kjpindex)                      :: zcoastal      !! Work array to keep coastal flow
    REAL(stnd),DIMENSION (kjpindex)                      :: zriver        !! Work array to keep river out flow
    REAL(stnd),DIMENSION (kjpindex)                      :: dcoastal      !! Work array to keep coastal flow
    REAL(stnd),DIMENSION (kjpindex)                      :: driver        !! Work array to keep river out flow
    REAL(stnd),DIMENSION (kjpindex)                      :: ztsol_rad     !! Work array to keep tsol_rad
    REAL(stnd),DIMENSION (kjpindex)                      :: zvevapp       !! Work array to keep vevapp
    REAL(stnd),DIMENSION (kjpindex)                      :: ztemp_sol_new !! Work array to keep temp_sol_new
    REAL(stnd),DIMENSION (kjpindex)                      :: zqsurf        !! Work array to keep qsurf
    REAL(stnd),DIMENSION (kjpindex,2)                    :: zalbedo       !! Work array to keep albedo
    REAL(stnd),DIMENSION (kjpindex)                      :: zfluxsens     !! Work array to keep fluxsens
    REAL(stnd),DIMENSION (kjpindex)                      :: zfluxlat      !! Work array to keep fluxlat
    REAL(stnd),DIMENSION (kjpindex)                      :: zemis         !! Work array to keep emis
    !
    ! Local but with input shape
    !
    REAL(stnd),DIMENSION (iim*jjm)                       :: dswnet         !! Net surface short-wave flux
    !
    INTEGER(long)                                        :: i, j, ik
    INTEGER(long)                                        :: itau_sechiba
    REAL(stnd), ALLOCATABLE, DIMENSION(:,:)              :: tmp_lon, tmp_lat
    LOGICAL :: check = .FALSE.

    IF (l_first_intersurf) THEN
       !
       IF ( check ) WRITE(*,*) 'Initialisation of intersurf'
       !
       !  Configuration of SSL specific parameters
       !
       CALL intsurf_config(control_flags)
       !
       !  Create the internal coordinate table
       !
       IF ( (.NOT.ALLOCATED(tmp_lon))) THEN
          ALLOCATE(tmp_lon(iim,jjm))
       ENDIF
       IF ( (.NOT.ALLOCATED(tmp_lat))) THEN
          ALLOCATE(tmp_lat(iim,jjm))
       ENDIF
       !
       IF ( (.NOT.ALLOCATED(lalo))) THEN
          ALLOCATE(lalo(kjpindex,2))
       ENDIF
       !
       DO i=1,iim
          DO j=1,jjm
             ik = (j-1)*iim + i
             tmp_lon(i,j) = lon(ik)
             tmp_lat(i,j) = lat(ik)
          ENDDO
       ENDDO
       !
       lalo(:,1) = lat(:)
       lalo(:,2) = lon(:)
       !-
       !- Compute variable to help describe the grid
       !- once the points are gathered.
       !-
       IF ( (.NOT.ALLOCATED(neighbours))) THEN
          ALLOCATE(neighbours(kjpindex,8))
       ENDIF
       IF ( (.NOT.ALLOCATED(resolution))) THEN
          ALLOCATE(resolution(kjpindex,2))
       ENDIF
       !
       CALL grid_stuff (kjpindex, iim, jjm, tmp_lon, tmp_lat, kindex, neighbours, resolution)
       !
       !- Store the fraction of the continents only once so that the user
       !- does not change them afterwards.
       !
       IF ( (.NOT.ALLOCATED(contfrac))) THEN
          ALLOCATE(contfrac(kjpindex))
       ENDIF
       !
       DO ik=1,kjpindex

          contfrac(ik) = zcontfrac(kindex(ik))

       ENDDO
       !
       CALL intsurf_restart(kjit, iim, jjm, tmp_lon, tmp_lat, date0, xrdt, control_flags, rest_id, rest_id_stom, itau_offset)
       !
       CALL intsurf_history(iim, jjm, tmp_lon, tmp_lat, kjit+itau_offset, date0_shifted, xrdt, control_flags, hist_id, hist_id_stom)
       !
       IF ( watchout ) THEN

          WRITE(*,*) 'intersurf_main_1d : WARNING -- ORCHIDEE_watchout has not yet been '
          WRITE(*,*) '                               implemented in this interface to ORCHIDEE'

       ENDIF
       !
       IF ( check ) WRITE(*,*) 'End of Initialisation of intersurf'
       !
    ENDIF

    !
    ! 1. gather input fields from kindex array
    !
    DO ik=1, kjpindex
       
       zu(ik)           = u(kindex(ik))
       zv(ik)           = v(kindex(ik))
       zzlev(ik)        = zlev(kindex(ik))
       zqair(ik)        = qair(kindex(ik))
       zprecip_rain(ik) = precip_rain(kindex(ik))*xrdt
       zprecip_snow(ik) = precip_snow(kindex(ik))*xrdt
       zlwdown(ik)      = lwdown(kindex(ik))
       zswnet(ik)       = swnet(kindex(ik))
       zswdown(ik)      = swdown(kindex(ik))
       ztemp_air(ik)    = temp_air(kindex(ik))
       zepot_air(ik)    = epot_air(kindex(ik))
       zccanopy(ik)     = ccanopy(kindex(ik))
       zpetAcoef(ik)    = petAcoef(kindex(ik))
       zpeqAcoef(ik)    = peqAcoef(kindex(ik))
       zpetBcoef(ik)    = petBcoef(kindex(ik))
       zpeqBcoef(ik)    = peqBcoef(kindex(ik))
       zcdrag(ik)       = cdrag(kindex(ik))
       zpb(ik)          = pb(kindex(ik))
       
    ENDDO
    !
    !
    !
    ! 3. call sechiba for continental points only
    !
    IF ( check ) WRITE(*,*) 'Calling sechiba'
    !
    !  Shift the time step to phase the two models
    !
    itau_sechiba = kjit + itau_offset
    !
    CALL sechiba_main (itau_sechiba, iim*jjm, kjpindex, kindex, xrdt, date0_shifted, &
       & lrestart_read, lrestart_write, control_flags, &
       & lalo, contfrac, neighbours, resolution, &
! First level conditions
       & zzlev, zu, zv, zqair, ztemp_air, zepot_air, zccanopy, &
! Variables for the implicit coupling
       & zcdrag, zpetAcoef, zpeqAcoef, zpetBcoef, zpeqBcoef, &
! Rain, snow, radiation and surface pressure
       & zprecip_rain ,zprecip_snow,  zlwdown, zswnet, zswdown, zpb, &
! Output : Fluxes
       & zvevapp, zfluxsens, zfluxlat, zcoastal, zriver, &
! Surface temperatures and surface properties
       & ztsol_rad, ztemp_sol_new, zqsurf, zalbedo, zemis, zz0, &
! File ids
       & rest_id, hist_id, rest_id_stom, hist_id_stom ) 
    
    !
    IF ( check ) WRITE(*,*) 'out of SECHIBA'
    !
    ! 4. scatter output fields
    !
    !
    z0(:)           = undef_sechiba
    coastalflow(:)  = undef_sechiba
    riverflow(:)    = undef_sechiba
    tsol_rad(:)     = undef_sechiba
    vevapp(:)       = undef_sechiba
    temp_sol_new(:) = undef_sechiba 
    qsurf(:)        = undef_sechiba 
    albedo(:,:)     = undef_sechiba
    albedo(:,:)     = undef_sechiba
    fluxsens(:)     = undef_sechiba
    fluxlat(:)      = undef_sechiba
    emis(:)         = undef_sechiba 
    cdrag(:)        = undef_sechiba 
    dswnet(:)       = undef_sechiba 
    !
    DO ik=1, kjpindex
       
       z0(kindex(ik))           = zz0(ik)
       coastalflow(kindex(ik))  = zcoastal(ik)/1000.
       riverflow(kindex(ik))    = zriver(ik)/1000.
       tsol_rad(kindex(ik))     = ztsol_rad(ik)
       vevapp(kindex(ik))       = zvevapp(ik)
       temp_sol_new(kindex(ik)) = ztemp_sol_new(ik)
       qsurf(kindex(ik))        = zqsurf(ik)
       albedo(kindex(ik),1)     = zalbedo(ik,1)
       albedo(kindex(ik),2)     = zalbedo(ik,2)
       fluxsens(kindex(ik))     = zfluxsens(ik)
       fluxlat(kindex(ik))      = zfluxlat(ik)
       emis(kindex(ik))         = zemis(ik)
       cdrag(kindex(ik))        = zcdrag(ik)
       dswnet(kindex(ik))        = zswnet(ik)

    ENDDO
    !
    !
    ! Modified fields for variables scattered during the writing
    !
    dcoastal(:) = (zcoastal(:))/1000.
    driver(:)   = (zriver(:))/1000.
    !
    !
    IF ( .NOT. l_first_intersurf) THEN
       !
       IF ( .NOT. almaoutput ) THEN
          !
          !  scattered during the writing
          !
          CALL histwrite (hist_id, 'evap',     itau_sechiba, zvevapp, kjpindex, kindex)
          CALL histwrite (hist_id, 'coastalflow',itau_sechiba, dcoastal, kjpindex, kindex)
          CALL histwrite (hist_id, 'riverflow',itau_sechiba, driver, kjpindex, kindex)
          !
          CALL histwrite (hist_id, 'temp_sol', itau_sechiba, temp_sol_NEW, iim*jjm, kindex)
          CALL histwrite (hist_id, 'tsol_max', itau_sechiba, temp_sol_NEW, iim*jjm, kindex)
          CALL histwrite (hist_id, 'tsol_min', itau_sechiba, temp_sol_NEW, iim*jjm, kindex)
          CALL histwrite (hist_id, 'fluxsens', itau_sechiba, fluxsens, iim*jjm, kindex)
          CALL histwrite (hist_id, 'fluxlat',  itau_sechiba, fluxlat, iim*jjm, kindex)
          CALL histwrite (hist_id, 'swnet',    itau_sechiba, dswnet, iim*jjm, kindex)
          CALL histwrite (hist_id, 'alb_vis',  itau_sechiba, albedo(:,1), iim*jjm, kindex)
          CALL histwrite (hist_id, 'alb_nir',  itau_sechiba, albedo(:,2), iim*jjm, kindex)
          CALL histwrite (hist_id, 'tair',     itau_sechiba, temp_air, iim*jjm, kindex)
          CALL histwrite (hist_id, 'qair',     itau_sechiba, qair, iim*jjm, kindex)
       ELSE
          CALL histwrite (hist_id, 'Evap', itau_sechiba, zvevapp, kjpindex, kindex)
          CALL histwrite (hist_id, 'SWnet',    itau_sechiba, dswnet, iim*jjm, kindex)
          CALL histwrite (hist_id, 'Qh', itau_sechiba, fluxsens, iim*jjm, kindex)
          CALL histwrite (hist_id, 'Qle',  itau_sechiba, fluxlat, iim*jjm, kindex)
          CALL histwrite (hist_id, 'AvgSurfT', itau_sechiba, temp_sol_NEW, iim*jjm, kindex)
          CALL histwrite (hist_id, 'RadT', itau_sechiba, temp_sol_NEW, iim*jjm, kindex)
       ENDIF
       !
    ENDIF
    !
    ! 5.0 Transform the water fluxes into Kg/m^2s and m^3/s
    !
    DO ik=1, kjpindex

       vevapp(kindex(ik)) = vevapp(kindex(ik))/xrdt
       coastalflow(kindex(ik)) = coastalflow(kindex(ik))/xrdt
       riverflow(kindex(ik)) = riverflow(kindex(ik))/xrdt

    ENDDO
    !
    l_first_intersurf = .FALSE.
    !
    IF (long_print) WRITE (numout,*) ' intersurf_main done '
    
  END SUBROUTINE intersurf_main_1d
!
!-------------------------------------------------------------------------------------
!
  SUBROUTINE intersurf_gathered (kjit, iim, jjm, kjpindex, kindex, xrdt, &
     & lrestart_read, lrestart_write, latlon, zcontfrac, zneighbours, zresolution, date0, &
! First level conditions
     & zlev,  u, v, qair, temp_air, epot_air, ccanopy, &
! Variables for the implicit coupling
     & cdrag, petAcoef, peqAcoef, petBcoef, peqBcoef, &
! Rain, snow, radiation and surface pressure
     & precip_rain, precip_snow, lwdown, swnet, swdown, pb, &
! Output : Fluxes
     & vevapp, fluxsens, fluxlat, coastalflow, riverflow, &
! Surface temperatures and surface properties
     & tsol_rad, temp_sol_new, qsurf, albedo, emis, z0, lon_scat, lat_scat) 
    
    ! routines called : sechiba_main
    !
    IMPLICIT NONE
    !    
    ! interface description for dummy arguments
    ! input scalar 
    INTEGER(long),INTENT (in)                            :: kjit          !! Time step number
    INTEGER(long),INTENT (in)                            :: iim, jjm      !! Dimension of input fields
    INTEGER(long),INTENT (in)                            :: kjpindex      !! Number of continental points
    REAL(stnd),INTENT (in)                               :: xrdt          !! Time step in seconds
    LOGICAL, INTENT (in)                                 :: lrestart_read !! Logical for _restart_ file to read
    LOGICAL, INTENT (in)                                 :: lrestart_write!! Logical for _restart_ file to write'
    REAL(stnd), INTENT (in)                              :: date0         !! Date at which kjit = 0
    ! input fields
    INTEGER(long),DIMENSION (kjpindex), INTENT (in)      :: kindex        !! Index for continental points
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)          :: u             !! Lowest level wind speed
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)          :: v             !! Lowest level wind speed 
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)          :: zlev          !! Height of first layer
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)          :: qair          !! Lowest level specific humidity
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)          :: precip_rain   !! Rain precipitation
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)          :: precip_snow   !! Snow precipitation
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)          :: lwdown        !! Down-welling long-wave flux 
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)          :: swnet         !! Net surface short-wave flux
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)          :: swdown        !! Downwelling surface short-wave flux
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)          :: temp_air      !! Air temperature in Kelvin
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)          :: epot_air      !! Air humidity
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)          :: ccanopy       !! CO2 concentration in the canopy
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)          :: petAcoef      !! Coeficients A from the PBL resolution
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)          :: peqAcoef      !! One for T and another for q
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)          :: petBcoef      !! Coeficients B from the PBL resolution
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)          :: peqBcoef      !! One for T and another for q
    REAL(stnd),DIMENSION (kjpindex), INTENT(inout)       :: cdrag         !! Cdrag
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)          :: pb            !! Lowest level pressure
    REAL(stnd),DIMENSION (kjpindex,2), INTENT(in)        :: latlon        !! Geographical coordinates
    REAL(stnd),DIMENSION (kjpindex), INTENT(in)          :: zcontfrac     !! Fraction of continent
    INTEGER(long),DIMENSION (kjpindex,8), INTENT(in)     :: zneighbours   !! neigbours
    REAL(stnd),DIMENSION (kjpindex,2), INTENT(in)        :: zresolution   !! size of the grid box
    ! output fields
    REAL(stnd),DIMENSION (kjpindex), INTENT(out)         :: z0            !! Surface roughness
    REAL(stnd),DIMENSION (kjpindex), INTENT(out)         :: coastalflow   !! Diffuse flow of water into the ocean (m^3/dt)
    REAL(stnd),DIMENSION (kjpindex), INTENT(out)         :: riverflow     !! Largest rivers flowing into the ocean (m^3/dt)
    REAL(stnd),DIMENSION (kjpindex), INTENT(out)         :: tsol_rad      !!
    REAL(stnd),DIMENSION (kjpindex), INTENT(out)         :: vevapp        !! Total of evaporation
    REAL(stnd),DIMENSION (kjpindex), INTENT(out)         :: temp_sol_new  !! New soil temperature
    REAL(stnd),DIMENSION (kjpindex), INTENT(out)         :: qsurf         !! Surface specific humidity
    REAL(stnd),DIMENSION (kjpindex,2), INTENT(out)       :: albedo        !! Albedo
    REAL(stnd),DIMENSION (kjpindex), INTENT(out)         :: fluxsens      !! Sensible chaleur flux
    REAL(stnd),DIMENSION (kjpindex), INTENT(out)         :: fluxlat       !! Latent chaleur flux
    REAL(stnd),DIMENSION (kjpindex), INTENT(out)         :: emis          !! Emissivity
    ! LOCAL declaration
    ! work arrays to scatter and/or gather information just before/after sechiba_main call's
    ! and to keep output value for next call
    REAL(stnd),DIMENSION (kjpindex)                      :: zprecip_rain  !! Work array to keep precip_rain
    REAL(stnd),DIMENSION (kjpindex)                      :: zprecip_snow  !! Work array to keep precip_snow
    REAL(stnd),DIMENSION (kjpindex)                      :: zz0           !! Work array to keep z0
    REAL(stnd),DIMENSION (kjpindex)                      :: zcdrag        !! Work array for surface drag
    REAL(stnd),DIMENSION (kjpindex)                      :: zcoastal      !! Work array to keep coastal flow
    REAL(stnd),DIMENSION (kjpindex)                      :: zriver        !! Work array to keep river out flow
    REAL(stnd),DIMENSION (kjpindex)                      :: dcoastal      !! Work array to keep coastal flow
    REAL(stnd),DIMENSION (kjpindex)                      :: driver        !! Work array to keep river out flow
    REAL(stnd),DIMENSION (kjpindex)                      :: ztsol_rad     !! Work array to keep tsol_rad
    REAL(stnd),DIMENSION (kjpindex)                      :: zvevapp       !! Work array to keep vevapp
    REAL(stnd),DIMENSION (kjpindex)                      :: ztemp_sol_new !! Work array to keep temp_sol_new
    REAL(stnd),DIMENSION (kjpindex)                      :: zqsurf        !! Work array to keep qsurf
    REAL(stnd),DIMENSION (kjpindex,2)                    :: zalbedo       !! Work array to keep albedo
    REAL(stnd),DIMENSION (kjpindex)                      :: zfluxsens     !! Work array to keep fluxsens
    REAL(stnd),DIMENSION (kjpindex)                      :: zfluxlat      !! Work array to keep fluxlat
    REAL(stnd),DIMENSION (kjpindex)                      :: zemis         !! Work array to keep emis
    !
    ! Optional arguments
    !
    REAL(stnd),DIMENSION (iim,jjm), INTENT(IN), OPTIONAL :: lon_scat, lat_scat !! The scattered values for longitude 
    !                                                                          !! and latitude.
    !
    ! Scattered variables for diagnostics
    !
    REAL(stnd),DIMENSION (iim*jjm)                       :: dvevapp       !! Diagnostic array for evaporation
    REAL(stnd),DIMENSION (iim*jjm)                       :: dtemp_sol     !! for surface temperature
    REAL(stnd),DIMENSION (iim*jjm)                       :: dfluxsens     !! for sensible heat flux
    REAL(stnd),DIMENSION (iim*jjm)                       :: dfluxlat      !! for latent heat flux
    REAL(stnd),DIMENSION (iim*jjm)                       :: dswnet        !! net solar radiation
    REAL(stnd),DIMENSION (iim*jjm,2)                     :: dalbedo       !! albedo
    REAL(stnd),DIMENSION (iim*jjm)                       :: dtair         !! air temperature
    REAL(stnd),DIMENSION (iim*jjm)                       :: dqair         !! specific air humidity
    !
    !
    INTEGER(long)                                        :: i, j, ik
    INTEGER(long)                                        :: itau_sechiba
    REAL(stnd)                                           :: mx, zlev_mean
    REAL(stnd), ALLOCATABLE, DIMENSION(:,:)              :: tmp_lon, tmp_lat
    LOGICAL :: check = .FALSE.

    IF (l_first_intersurf) THEN

       !
       IF ( check ) WRITE(*,*) 'Initialisation of intersurf'
       !
       !  Configuration of SSL specific parameters
       !
       CALL intsurf_config(control_flags)
       !
       !  Create the internal coordinate table
       !
       IF ( (.NOT.ALLOCATED(lalo))) THEN
          ALLOCATE(lalo(kjpindex,2))
       ENDIF 
       !
       lalo(:,:) = latlon(:,:)
       !
       IF ( (.NOT.ALLOCATED(neighbours))) THEN
          ALLOCATE(neighbours(kjpindex,8))
       ENDIF
       neighbours(:,:) = zneighbours(:,:)
       !
       IF ( (.NOT.ALLOCATED(resolution))) THEN
          ALLOCATE(resolution(kjpindex,2))
       ENDIF
       resolution(:,:) = zresolution(:,:)
       !
       !- Store the fraction of the continents only once so that the user
       !- does not change them afterwards.
       !
       IF ( (.NOT.ALLOCATED(contfrac))) THEN
          ALLOCATE(contfrac(kjpindex))
       ENDIF
        contfrac(:) = zcontfrac(:)
       !
       !
       !  Create the internal coordinate table
       !
       IF ( (.NOT.ALLOCATED(tmp_lon))) THEN
          ALLOCATE(tmp_lon(iim,jjm))
       ENDIF
       IF ( (.NOT.ALLOCATED(tmp_lat))) THEN
          ALLOCATE(tmp_lat(iim,jjm))
       ENDIF
       !
       !  Either we have the scattered coordinates as arguments or
       !  we have to do the work here.
       !
       IF ( PRESENT(lon_scat) .AND. PRESENT(lat_scat)) THEN
          
          tmp_lon(:,:) = lon_scat(:,:)
          tmp_lat(:,:) = lat_scat(:,:)

       ELSE IF ( PRESENT(lon_scat) .OR. PRESENT(lat_scat)) THEN

          WRITE(*,*) 'You need to provide the longitude AND latitude on the'
          WRITE(*,*) 'gathered grid in order to start ORCHIDEE.'
          STOP 'intersurf_gathered'

       ELSE
          !
          WRITE(*,*) 'intersurf_gathered : We try to guess to full grid of the model.' 
          WRITE(*,*) 'I might fail, please report if it does. '
          !
          tmp_lon(:,:) = val_exp
          tmp_lat(:,:) = val_exp
          !
          DO ik=1, kjpindex
             j = INT( (kindex(ik)-1) / iim ) + 1
             i = kindex(ik) - (j-1) * iim
             tmp_lon(i,j) = lalo(ik,2)
             tmp_lat(i,j) = lalo(ik,1)
          ENDDO
          !
          ! Here we fill out the grid. To do this we do the strong hypothesis
          ! that the grid is regular. Will this work in all cases ????
          !
          DO i=1,iim
             mx = MAXVAL(tmp_lon(i,:), MASK=tmp_lon(i,:) .LT. val_exp)
             IF ( mx .LT. val_exp ) THEN
                tmp_lon(i,:) = mx
             ELSE
                WRITE(*,*) 'Could not find a continental point on this longitude. Thus the grid'
                WRITE(*,*) 'could not be completed.'
                STOP 'intersurf_gathered'
             ENDIF
          ENDDO
          !
          DO j=1,jjm
             mx = MAXVAL(tmp_lat(:,j), MASK=tmp_lat(:,j) .LT. val_exp)
             IF ( mx .LT. val_exp ) THEN
                tmp_lat(:,j) = mx
             ELSE
                WRITE(*,*) 'Could not find a continental point on this latitude. Thus the grid'
                WRITE(*,*) 'could not be completed.'
                STOP 'intersurf_gathered'
             ENDIF
          ENDDO
       ENDIF
       !
       !
       CALL intsurf_restart(kjit, iim, jjm, tmp_lon, tmp_lat, date0, xrdt, control_flags, rest_id, rest_id_stom, itau_offset)
       !
       CALL intsurf_history(iim, jjm, tmp_lon, tmp_lat, kjit+itau_offset, date0_shifted, xrdt, control_flags, hist_id, hist_id_stom)
       !
       IF ( watchout ) THEN

          zlev_mean = SUM(zlev)/REAL(kjpindex,stnd)

          CALL intersurf_initwatch(iim, jjm, kjpindex, date0, kjit+itau_offset, xrdt, kindex, tmp_lon, tmp_lat, zlev_mean, contfrac)
          
       ENDIF
       !
       IF ( check ) WRITE(*,*) 'End of Initialisation of intersurf'
       !
    ENDIF

    !
    ! 1. Just change the units of some input fields
    !
    DO ik=1, kjpindex
       
       zprecip_rain(ik) = precip_rain(ik)*xrdt
       zprecip_snow(ik) = precip_snow(ik)*xrdt
       zcdrag(ik)       = cdrag(ik)
       
    ENDDO
    !
    !
    !
    ! 3. call sechiba for continental points only
    !
    IF ( check ) WRITE(*,*) 'Calling sechiba'
    !
    !  Shift the time step to phase the two models
    !
    itau_sechiba = kjit + itau_offset
    !
    IF ( watchout ) THEN

       IF ( .NOT. l_first_intersurf) THEN
          CALL intersurf_wrtwatch(kjpindex, itau_sechiba, xrdt, swdown, zprecip_rain, &
               &    zprecip_snow, lwdown, pb, temp_air, qair, u, v)
       ENDIF
       
    ENDIF
    !
    CALL sechiba_main (itau_sechiba, iim*jjm, kjpindex, kindex, xrdt, date0_shifted, &
       & lrestart_read, lrestart_write, control_flags, &
       & lalo, contfrac, neighbours, resolution, &
! First level conditions
       & zlev, u, v, qair, temp_air, epot_air, ccanopy, &
! Variables for the implicit coupling
       & zcdrag, petAcoef, peqAcoef, petBcoef, peqBcoef, &
! Rain, snow, radiation and surface pressure
       & zprecip_rain ,zprecip_snow,  lwdown, swnet, swdown, pb, &
! Output : Fluxes
       & zvevapp, zfluxsens, zfluxlat, zcoastal, zriver, &
! Surface temperatures and surface properties
       & ztsol_rad, ztemp_sol_new, zqsurf, zalbedo, zemis, zz0, &
! File ids
       & rest_id, hist_id, rest_id_stom, hist_id_stom ) 
    
    !
    IF ( check ) WRITE(*,*) 'out of SECHIBA'
    !
    ! 4. scatter output fields
    !
    z0(:)           = undef_sechiba
    coastalflow(:)  = undef_sechiba
    riverflow(:)    = undef_sechiba
    tsol_rad(:)     = undef_sechiba
    vevapp(:)       = undef_sechiba
    temp_sol_new(:) = undef_sechiba
    qsurf(:)        = undef_sechiba
    albedo(:,1)     = undef_sechiba
    albedo(:,2)     = undef_sechiba
    fluxsens(:)     = undef_sechiba
    fluxlat(:)      = undef_sechiba
    emis(:)         = undef_sechiba
    cdrag(:)        = undef_sechiba
    !    
    dvevapp(:)    = undef_sechiba
    dtemp_sol(:)  = undef_sechiba
    dfluxsens(:)  = undef_sechiba
    dfluxlat(:)   = undef_sechiba
    dswnet (:)    = undef_sechiba
    dalbedo (:,1) = undef_sechiba
    dalbedo (:,2) = undef_sechiba
    dtair (:)     = undef_sechiba
    dqair (:)     = undef_sechiba
    !
    DO ik=1, kjpindex
       
       z0(ik)           = zz0(ik)
       coastalflow(ik)  = zcoastal(ik)/1000.
       riverflow(ik)    = zriver(ik)/1000.
       tsol_rad(ik)     = ztsol_rad(ik)
       vevapp(ik)       = zvevapp(ik)
       temp_sol_new(ik) = ztemp_sol_new(ik)
       qsurf(ik)        = zqsurf(ik)
       albedo(ik,1)     = zalbedo(ik,1)
       albedo(ik,2)     = zalbedo(ik,2)
       fluxsens(ik)     = zfluxsens(ik)
       fluxlat(ik)      = zfluxlat(ik)
       emis(ik)         = zemis(ik)
       cdrag(ik)        = zcdrag(ik)
       
       ! Fill up the disgnostic arrays

       dvevapp(kindex(ik))    = zvevapp(ik)
       dtemp_sol(kindex(ik))  = ztemp_sol_new(ik)
       dfluxsens(kindex(ik))  = zfluxsens(ik)
       dfluxlat(kindex(ik))   = zfluxlat(ik)
       dswnet (kindex(ik))    = swnet(ik)
       dalbedo (kindex(ik),1) = zalbedo(ik,1)
       dalbedo (kindex(ik),2) = zalbedo(ik,2)   
       dtair (kindex(ik))     = temp_air(ik)
       dqair (kindex(ik))     = qair(ik)
       !
    ENDDO
    ! Modified fields for variables scattered during the writing
    !
    dcoastal(:) = (zcoastal(:))/1000.
    driver(:)   = (zriver(:))/1000.
    !
    !
    !
    !
    IF ( .NOT. l_first_intersurf) THEN
       !
       IF ( .NOT. almaoutput ) THEN
          !
          !  scattered during the writing
          !           
          CALL histwrite (hist_id, 'evap',     itau_sechiba, zvevapp, kjpindex, kindex)
          CALL histwrite (hist_id, 'coastalflow',itau_sechiba, dcoastal, kjpindex, kindex)
          CALL histwrite (hist_id, 'riverflow',itau_sechiba, driver, kjpindex, kindex)
          ! 
          CALL histwrite (hist_id, 'temp_sol', itau_sechiba, dtemp_sol, iim*jjm, kindex)
          CALL histwrite (hist_id, 'tsol_max', itau_sechiba, dtemp_sol, iim*jjm, kindex)
          CALL histwrite (hist_id, 'tsol_min', itau_sechiba, dtemp_sol, iim*jjm, kindex)
          CALL histwrite (hist_id, 'fluxsens', itau_sechiba, dfluxsens, iim*jjm, kindex)
          CALL histwrite (hist_id, 'fluxlat', itau_sechiba, dfluxlat, iim*jjm, kindex)
          CALL histwrite (hist_id, 'swnet',    itau_sechiba, dswnet, iim*jjm, kindex)
          CALL histwrite (hist_id, 'alb_vis',  itau_sechiba, dalbedo(:,1), iim*jjm, kindex)
          CALL histwrite (hist_id, 'alb_nir',  itau_sechiba, dalbedo(:,2), iim*jjm, kindex)
          CALL histwrite (hist_id, 'tair',     itau_sechiba, dtair, iim*jjm, kindex)
          CALL histwrite (hist_id, 'qair',     itau_sechiba, dqair, iim*jjm, kindex)
          !
       ELSE
          !
          CALL histwrite (hist_id, 'Evap', itau_sechiba, zvevapp, kjpindex, kindex)
          CALL histwrite (hist_id, 'SWnet',    itau_sechiba, dswnet, iim*jjm, kindex)
          CALL histwrite (hist_id, 'Qh', itau_sechiba, dfluxsens, iim*jjm, kindex)
          CALL histwrite (hist_id, 'Qle',  itau_sechiba, dfluxlat, iim*jjm, kindex)
          CALL histwrite (hist_id, 'Evap', itau_sechiba, zvevapp, iim*jjm, kindex)
          CALL histwrite (hist_id, 'AvgSurfT', itau_sechiba, dtemp_sol, iim*jjm, kindex)
          CALL histwrite (hist_id, 'RadT', itau_sechiba, dtemp_sol, iim*jjm, kindex)
          !
       ENDIF
       !
    ENDIF
    !
    ! 5.0 Transform the water fluxes into Kg/m^2s and m^3/s
    !
    DO ik=1, kjpindex

       vevapp(ik) = vevapp(ik)/xrdt
       coastalflow(ik) = coastalflow(ik)/xrdt
       riverflow(ik) = riverflow(ik)/xrdt

    ENDDO
    !
    IF ( lrestart_write .AND. watchout ) THEN

       CALL intersurf_clowatch()

    ENDIF
    !
    l_first_intersurf = .FALSE.
    !
    IF (long_print) WRITE (numout,*) ' intersurf_main done '
    
  END SUBROUTINE intersurf_gathered
!

!-------------------------------------------------------------------------------------
!
  SUBROUTINE intsurf_config(control_flags)
    !
    !  This subroutine reads all the configuration flags which control the behaviour of the model
    !
    TYPE(control_type), INTENT(out)            :: control_flags !! Flags that (de)activate parts of the model
    !
    !
    !Config Key  = ORCHIDEE_WATCHOUT
    !Config Desc = ORCHIDEE will write out its forcing to a file
    !Config Def  = n
    !Config Help = This flag allows to write to a file all the variables
    !Config        which are used to force the land-surface. The file 
    !Config        has exactly the same format than a normal off-line forcing
    !Config        and thus this forcing can be used for forcing ORCHIDEE.
    !
    watchout = .FALSE.
    CALL getin('ORCHIDEE_WATCHOUT',watchout)
    !
    !Config Key  = WATCHOUT_FILE
    !Config Desc = Filenane for the ORCHIDEE forcing file
    !Config IF   = ORCHIDEE_WATCHOUT
    !Config Def  = orchidee_watchout.nc
    !Config Help = This is the name of the file in which the
    !Config        forcing used here will be written for later use. 
    !
    watchout_file = "orchidee_watchout.nc"
    CALL getin('WATCHOUT_FILE',watchout_file)

    WRITE(*,*) 'WATCHOUT flag :', watchout
    WRITE(*,*) 'WATCHOUT file :', watchout_file
    !
    !Config Key  = STOMATE_OK_CO2
    !Config Desc = Activate CO2?
    !Config Def  = n
    !Config Help = set to TRUE if photosynthesis is to be activated
    !
    control_flags%ok_co2 = .FALSE.
    CALL getin('STOMATE_OK_CO2', control_flags%ok_co2)
    WRITE(*,*) 'photosynthesis: ', control_flags%ok_co2
    !
    !Config Key  = STOMATE_OK_STOMATE
    !Config Desc = Activate STOMATE?
    !Config Def  = n
    !Config Help = set to TRUE if STOMATE is to be activated
    !
    control_flags%ok_stomate = .FALSE.
    CALL getin('STOMATE_OK_STOMATE',control_flags%ok_stomate)
    WRITE(*,*) 'STOMATE is activated: ',control_flags%ok_stomate
    !
    !Config Key  = STOMATE_OK_DGVM
    !Config Desc = Activate DGVM?
    !Config Def  = n
    !Config Help = set to TRUE if DGVM is to be activated
    !
    control_flags%ok_dgvm = .FALSE.
    CALL getin('STOMATE_OK_DGVM',control_flags%ok_dgvm)
    WRITE(*,*) 'LPJ is activated: ',control_flags%ok_dgvm
    !
    ! Ensure consistency
    !
    IF ( control_flags%ok_dgvm ) control_flags%ok_stomate = .TRUE.
    IF ( control_flags%ok_stomate ) control_flags%ok_co2 = .TRUE.
    !
    !Config Key  = STOMATE_WATCHOUT
    !Config Desc = STOMATE does minimum service
    !Config Def  = n
    !Config Help = set to TRUE if you want STOMATE to read
    !Config        and write its start files and keep track
    !Config        of longer-term biometeorological variables.
    !Config        This is useful if OK_STOMATE is not set,
    !Config        but if you intend to activate STOMATE later.
    !Config        In that case, this run can serve as a 
    !Config        spinup for longer-term biometeorological
    !Config        variables.
    !
    control_flags%stomate_watchout = .FALSE.
    CALL getin('STOMATE_WATCHOUT',control_flags%stomate_watchout)
    WRITE(*,*) 'STOMATE keeps an eye open: ',control_flags%stomate_watchout
    !
    ! Here we need the same initialisation as above
    !
    control_flags%ok_pheno = .TRUE.
    !
    !
    RETURN
    !
  END SUBROUTINE intsurf_config
  !
  !
  !
  SUBROUTINE intsurf_restart(istp, iim, jjm, lon, lat, date0, dt, control_flags, rest_id, rest_id_stom, itau_offset)
    !
    !  This subroutine initialized the restart file for the land-surface scheme
    !
    INTEGER(long), INTENT(in)                  :: istp      !! Time step of the restart file
    INTEGER(long), INTENT(in)                  :: iim, jjm  !! Size in x and y of the data to be handeled
    REAL(stnd),DIMENSION (iim,jjm), INTENT(in) :: lon, lat  !! Logitude and latitude of the data points
    REAL(stnd)                                 :: date0     !! The date at which itau = 0
    REAL(stnd)                                 :: dt        !! Time step
    INTEGER(long), INTENT(out)                 :: rest_id, rest_id_stom   !! ID of the restart file
    INTEGER(long), INTENT(out)                 :: itau_offset
    !
    TYPE(control_type), INTENT(in)             :: control_flags !! Flags that (de)activate parts of the model
    !
    !  LOCAL
    !
    CHARACTER(LEN=80)          :: restname_in, restname_out, stom_restname_in, stom_restname_out
    REAL(stnd)                 :: dt_rest, date0_rest
    INTEGER(long)              :: itau_dep
    INTEGER(long),PARAMETER    :: llm=1
    REAL(stnd), DIMENSION(llm) :: lev
    LOGICAL                    :: overwrite_time
    REAL(stnd)                 :: in_julian, rest_julian
    REAL(stnd)                 :: un_an, un_jour
    INTEGER(long)              :: yy, mm, dd
    REAL(stnd)                 :: ss
    !
    !Config  Key  = SECHIBA_restart_in
    !Config  Desc = Name of restart to READ for initial conditions
    !Config  Def  = NONE
    !Config  Help = This is the name of the file which will be opened
    !Config         to extract the initial values of all prognostic
    !Config         values of the model. This has to be a netCDF file.
    !Config         Not truly COADS compliant. NONE will mean that
    !Config         no restart file is to be expected.
!-
    restname_in = 'NONE'
    CALL getin('SECHIBA_restart_in',restname_in)
    WRITE(*,*) 'INPUT RESTART_FILE', restname_in
    !-
    !Config Key  = SECHIBA_rest_out
    !Config Desc = Name of restart files to be created by SECHIBA
    !Config Def  = sechiba_rest_out.nc
    !Config Help = This variable give the name for
    !Config        the restart files. The restart software within
    !Config        IOIPSL will add .nc if needed.
    !
    restname_out = 'restart_out.nc'
    CALL getin('SECHIBA_rest_out', restname_out)
    !
    !Config Key  = SECHIBA_reset_time
    !Config Desc = Option to overrides the time of the restart
    !Config Def  = n
    !Config Help = This option allows the model to override the time
    !Config        found in the restart file of SECHIBA with the time
    !Config        of the first call. That is the restart time of the GCM.
    !
    overwrite_time = .FALSE.
    CALL getin('SECHIBA_reset_time', overwrite_time)
    !
    CALL ioget_calendar(un_an, un_jour)
    !
    lev(:) = 0.
    itau_dep = 0
    in_julian = itau2date(istp, date0, dt)
    date0_rest = date0
    dt_rest = dt
    !
    CALL restini( restname_in, iim, jjm, lon, lat, llm, lev, &
         &  restname_out, itau_dep, date0_rest, dt_rest, rest_id, overwrite_time)
    !
    !  itau_dep of SECHIBA is phased with the GCM if needed
    !
    rest_julian = itau2date(itau_dep, date0_rest, dt_rest)
    !
    IF ( ABS( in_julian - rest_julian) .GT. dt/un_jour ) THEN
       IF ( overwrite_time ) THEN
          WRITE(*,*) 'The SECHIBA restart is not for the same timestep as the GCM,'
          WRITE(*,*) 'the two are synchronized. The land-surface conditions can not impose'
          WRITE(*,*) 'the chronology of the simulation.'
          WRITE(*,*) 'Time step of the GCM :', istp, 'Julian day : ', in_julian
          CALL ju2ymds(in_julian, yy, mm, dd, ss)
          WRITE(*,*) 'In other word (yy,mm,dd,ss) : ', yy, mm, dd, ss
          WRITE(*,*) 'Time step of SECHIBA :', itau_dep, 'Julian day : ', rest_julian
          CALL ju2ymds(rest_julian, yy, mm, dd, ss)
          WRITE(*,*) 'In other word (yy,mm,dd,ss) : ', yy, mm, dd, ss

          itau_offset = itau_dep - istp
          date0_shifted = date0 - itau_offset*dt/un_jour

          WRITE(*,*) 'The new starting date is :', date0_shifted
          CALL ju2ymds(date0_shifted, yy, mm, dd, ss)
          WRITE(*,*) 'In other word (yy,mm,dd,ss) : ', yy, mm, dd, ss
       ELSE
          WRITE(*,*) 'IN -> OUT :', istp, '->', itau_dep
          WRITE(*,*) 'IN -> OUT :', in_julian, '->', rest_julian
          WRITE(*,*) 'SECHIBA''s restart file is not consistent with the one of the GCM'
          WRITE(*,*) 'Correct the time axis of the restart file or force the code to change it.'
          STOP
       ENDIF
    ELSE
       itau_offset = 0
       date0_shifted = date0
    ENDIF
    !
!!!    CALL ioconf_startdate(date0_shifted)
    !
    !=====================================================================
    !- 1.5 Restart file for STOMATE
    !=====================================================================
    IF ( control_flags%ok_stomate .OR. control_flags%stomate_watchout ) THEN 
       !-
       ! STOMATE IS ACTIVATED
       !-
       !Config  Key  = STOMATE_RESTART_FILEIN
       !Config  Desc = Name of restart to READ for initial conditions
       !Config         of STOMATE
       !Config  If   = STOMATE_OK_STOMATE || STOMATE_WATCHOUT
       !Config  Def  = NONE
       !Config  Help = This is the name of the file which will be opened
       !Config         to extract the initial values of all prognostic
       !Config         values of STOMATE.
       !-
       stom_restname_in = 'NONE'
       CALL getin('STOMATE_RESTART_FILEIN',stom_restname_in)
       WRITE(*,*) 'STOMATE INPUT RESTART_FILE', stom_restname_in
       !-
       !Config Key  = STOMATE_RESTART_FILEOUT
       !Config Desc = Name of restart files to be created by STOMATE
       !Config  If   = STOMATE_OK_STOMATE || STOMATE_WATCHOUT
       !Config Def  = stomate_restart.nc
       !Config Help = This is the name of the file which will be opened
       !Config        to write the final values of all prognostic values
       !Config        of STOMATE.
       !-
       stom_restname_out = 'stomate_restart.nc'
       CALL getin('STOMATE_RESTART_FILEOUT', stom_restname_out)
       WRITE(*,*) 'STOMATE OUTPUT RESTART_FILE', stom_restname_out
       !-
       CALL restini (stom_restname_in, iim, jjm, lon, lat, llm, lev, &
            &  stom_restname_out, itau_dep, date0_rest, dt_rest, rest_id_stom, overwrite_time)
       !-
    ENDIF
    !
  END SUBROUTINE intsurf_restart
  
  SUBROUTINE intsurf_history(iim, jjm, lon, lat, istp_old, date0, dt, control_flags, hist_id, hist_id_stom)
    !
    !   
    !  This subroutine initialized the history files for the land-surface scheme
    !
    !
    INTEGER(long), INTENT(in)                  :: iim, jjm  !! Size in x and y of the data to be handeled
    REAL(stnd),DIMENSION (iim,jjm), INTENT(in) :: lon, lat  !! Logitude and latitude of the data points
    INTEGER(long), INTENT(in)                  :: istp_old  !! Time step counter
    REAL(stnd), INTENT(in)                     :: date0     !! Julian day at which istp=0
    REAL(stnd), INTENT(in)                     :: dt        !! Time step of the counter in seconds
    !
    TYPE(control_type), INTENT(in)             :: control_flags !! Flags that (de)activate parts of the model
    !
    INTEGER(long), INTENT(out)                 :: hist_id, hist_id_stom !! History file identification for SECHIBA and STOMATE
    !
    !  LOCAL
    !
    CHARACTER(LEN=80) :: histname,stom_histname       !! Name of history file
    REAL(stnd)        :: dw                           !! frequency of history write (sec.)
    CHARACTER(LEN=30)   :: flux_op                    !! Operations to be performed on fluxes
    CHARACTER(LEN=30)   :: flux_sc                    !! Operations which do not include a scatter
    CHARACTER(LEN=30)   :: flux_insec, flux_scinsec   !! Operation in seconds
    INTEGER(long)     :: hist_level                   !! history output level (default is 10 => maximum output)
    CHARACTER(LEN=40),DIMENSION(max_hist_level) :: &
         & ave, avecels, avescatter, tmax, fluxop, fluxop_sc, fluxop_insec, &
         & fluxop_scinsec, tmincels, tmaxcels, once, sumscatter  !! The various operation to be performed
    INTEGER(long)     :: i
    INTEGER(long)     :: hori_id                      !! ID of the default horizontal longitude and latitude map.
    INTEGER(long)     :: vegax_id, solax_id, soltax_id, nobioax_id !! ID's for two vertical coordinates
    INTEGER(long)     :: hist_PFTaxis_id
    !
    LOGICAL                               :: rectilinear
    INTEGER(long)                         :: ier
    REAL(stnd), ALLOCATABLE, DIMENSION(:) :: lon_rect, lat_rect
    !
    REAL(stnd),DIMENSION(nvm)   :: veg
    REAL(stnd),DIMENSION(ngrnd) :: sol
    REAL(stnd),DIMENSION(nbsoil):: soltyp
    REAL(stnd),DIMENSION(nnobio):: nobiotyp
    !
    REAL(stnd)                  :: hist_days_stom     !!- GK time step in days for this history file
    REAL(stnd)                  :: hist_dt_stom       !!- GK time step in seconds for this history file
    INTEGER(long),PARAMETER     :: npft = nvm-1       !!- GK Number of PFTs
    REAL(stnd),DIMENSION(npft)  :: hist_PFTaxis       !!- GK An axis we need for the history files
    !
    !
    !=====================================================================
    !- 3.0 Setting up the history files
    !=====================================================================
    !- 3.1 SECHIBA
    !=====================================================================
    !Config  Key  = ALMA_OUTPUT
    !Config  Desc = Should the output follow the ALMA convention
    !Config  Def  = n
    !Config  Help = If this logical flag is set to true the model
    !Config         will output all its data according to the ALMA 
    !Config         convention. It is the recommended way to write
    !Config         data out of ORCHIDEE.
    !-
    almaoutput = .FALSE.
    CALL getin('ALMA_OUTPUT', almaoutput)
    WRITE(*,*) 'ALMA_OUTPUT', almaoutput
    !-
    !Config  Key  = OUTPUT_FILE
    !Config  Desc = Name of file in which the output is going
    !Config         to be written
    !Config  Def  = cabauw_out.nc
    !Config  Help = This file is going to be created by the model
    !Config         and will contain the output from the model.
    !Config         This file is a truly COADS compliant netCDF file.
    !Config         It will be generated by the hist software from
    !Config         the IOIPSL package.
    !-
    histname='cabauw_out.nc'
    CALL getin('OUTPUT_FILE', histname)
    WRITE(*,*) 'OUTPUT_FILE', histname
    !-
    !Config  Key  = WRITE_STEP
    !Config  Desc = Frequency in seconds at which to WRITE output
    !Config  Def  = 86400.0
    !Config  Help = This variables gives the frequency the output of
    !Config         the model should be written into the netCDF file.
    !Config         It does not affect the frequency at which the
    !Config         operations such as averaging are done.
    !Config         That is IF the coding of the calls to histdef
    !Config         are correct !
    !-
    dw = un_jour
    CALL getin('WRITE_STEP', dw)
    !-
    !
    veg(1:nvm)   = (/ (REAL(i,stnd),i=1,nvm) /)
    sol(1:ngrnd) = (/ (REAL(i,stnd),i=1,ngrnd) /)   
    soltyp(1:nbsoil) = (/ (REAL(i,stnd),i=1,nbsoil) /)
    nobiotyp(1:nnobio) = (/ (REAL(i,stnd),i=1,nnobio) /)
    !
    !- We need to flux averaging operation as when the data is written
    !- from within SECHIBA a scatter is needed. In the driver on the other
    !- hand the data is 2D and can be written is it is.
    !-
    WRITE(flux_op,'("ave(scatter(X*",F8.1,"))")') un_jour/dt
    ! WRITE(flux_op,'("(ave(scatter(X))*",F8.1,")")') un_jour/dt
    WRITE(flux_sc,'("ave(X*",F8.1,")")') un_jour/dt
    !WRITE(flux_sc,'("(ave(X)*",F8.1,")")') un_jour/dt
    WRITE(flux_insec,'("ave(X*",F8.6,")")') 1.0/dt
    WRITE(flux_scinsec,'("ave(scatter(X*",F8.6,"))")') 1.0/dt
    WRITE(*,*) flux_op, un_jour/dt, dt, dw
    !-
    !Config  Key  = SECHIBA_HISTLEVEL
    !Config  Desc = SECHIBA history output level (0..10)
    !Config  Def  = 5
    !Config  Help = Chooses the list of variables in the history file. 
    !Config         Values between 0: nothing is written; 10: everything is 
    !Config         written are available More details can be found on the web under documentation.
    !Config         web under documentation.
    !-
    hist_level = 5
    CALL getin('SECHIBA_HISTLEVEL', hist_level)
    !-
    WRITE(*,*) 'SECHIBA history level: ',hist_level
    IF ( (hist_level > max_hist_level).OR.(hist_level < 0) ) THEN
       STOP 'This history level is not allowed'
    ENDIF
    !-
    !- define operations as a function of history level.
    !- Above hist_level, operation='never'
    !-
    ave(1:10) = 'ave(X)'
    IF (hist_level < 10) THEN
       ave(hist_level+1:10) = 'never'
    ENDIF
    sumscatter(1:10) = 't_sum(scatter(X))'
    IF (hist_level < 10) THEN
       sumscatter(hist_level+1:10) = 'never'
    ENDIF
    avecels(1:10) = 'ave(cels(X))'
    IF (hist_level < 10) THEN
       avecels(hist_level+1:10) = 'never'
    ENDIF
    avescatter(1:10) = 'ave(scatter(X))'
    IF (hist_level < 10) THEN
       avescatter(hist_level+1:10) = 'never'
    ENDIF
    tmincels(1:10) = 't_min(cels(X))'
    IF (hist_level < 10) THEN
       tmincels(hist_level+1:10) = 'never'
    ENDIF
    tmaxcels(1:10) = 't_max(cels(X))'
    IF (hist_level < 10) THEN
       tmaxcels(hist_level+1:10) = 'never'
    ENDIF
    tmax(1:10) = 't_max(X)'
    IF (hist_level < 10) THEN
       tmax(hist_level+1:10) = 'never'
    ENDIF
    fluxop(1:10) = flux_op
    IF (hist_level < 10) THEN
       fluxop(hist_level+1:10) = 'never'
    ENDIF
    fluxop_sc(1:10) = flux_sc
    IF (hist_level < 10) THEN
       fluxop_sc(hist_level+1:10) = 'never'
    ENDIF
    fluxop_insec(1:10) = flux_insec
    IF (hist_level < 10) THEN
       fluxop_insec(hist_level+1:10) = 'never'
    ENDIF
    fluxop_scinsec(1:10) = flux_scinsec
    IF (hist_level < 10) THEN
       fluxop_scinsec(hist_level+1:10) = 'never'
    ENDIF
    once(1:10) = 'once(scatter(X))'
    IF (hist_level < 10) THEN
       once(hist_level+1:10) = 'never'
    ENDIF
    !-
    !- Check if we have by any change a rectilinear grid. This would allow us to 
    !- simplify the output files.
    !
    rectilinear = .FALSE.
    IF ( ALL(lon(:,:) == SPREAD(lon(:,1), 2, SIZE(lon,2))) .AND. &
       & ALL(lat(:,:) == SPREAD(lat(1,:), 1, SIZE(lat,1))) ) THEN
       rectilinear = .TRUE.
       ALLOCATE(lon_rect(iim),stat=ier)
       IF (ier .NE. 0) THEN
          WRITE (numout,*) ' error in lon_rect allocation. We stop. We need iim words = ',iim
          STOP 'intersurf_history'
       ENDIF
       ALLOCATE(lat_rect(jjm),stat=ier)
       IF (ier .NE. 0) THEN
          WRITE (numout,*) ' error in lat_rect allocation. We stop. We need jjm words = ',jjm
          STOP 'intersurf_history'
       ENDIF
       lon_rect(:) = lon(:,1)
       lat_rect(:) = lat(1,:)
    ENDIF
    !-
    !-
    !-
    IF ( .NOT. almaoutput ) THEN
       !- 
       IF ( rectilinear ) THEN
          CALL histbeg(histname, iim, lon_rect, jjm, lat_rect, 1, iim, 1, jjm, &
               &     istp_old, date0, dt, hori_id, hist_id)
       ELSE
          CALL histbeg(histname, iim, lon, jjm, lat, 1, iim, 1, jjm, &
               &     istp_old, date0, dt, hori_id, hist_id)
       ENDIF
       !-
       CALL histvert(hist_id, 'veget', 'Vegetation types', '-', &
            &    nvm,   veg, vegax_id)
       CALL histvert(hist_id, 'solth', 'Soil levels',      'm', &
            &    ngrnd, sol, solax_id)
       CALL histvert(hist_id, 'soiltyp', 'Soil types',      '-', &
            &    nbsoil, soltyp, soltax_id)
       CALL histvert(hist_id, 'nobio', 'Other surface types',      '-', &
            &    nnobio, nobiotyp, nobioax_id)
       !-
       !- SECHIBA_HISTLEVEL = 1
       !-
       CALL histdef(hist_id, 'evap', 'Evaporation', 'mm/d', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, fluxop(1), dt,dw)
       CALL histdef(hist_id, 'coastalflow', 'Diffuse coastal flow', 'm^3/s', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, fluxop_scinsec(1), dt,dw)
       CALL histdef(hist_id, 'riverflow', 'River flow to the oceans', 'm^3/s', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, fluxop_scinsec(1), dt,dw) 
       CALL histdef(hist_id, 'temp_sol', 'Surface Temperature', 'C', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avecels(1), dt,dw)
       CALL histdef(hist_id, 'rain', 'Rainfall', 'mm/d',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, fluxop(1), dt,dw)
       CALL histdef(hist_id, 'snowf', 'Snowfall', 'mm/d',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, fluxop(1), dt,dw)
       CALL histdef(hist_id, 'netrad', 'Net radiation', 'W/m^2',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(1), dt,dw)
       CALL histdef(hist_id, 'basinmap', 'Aproximate map of the river basins', ' ', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, once(1), dt,dw) 
       CALL histdef(hist_id, 'nbrivers', 'Number or rivers in the outflow grid box', ' ', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, once(1), dt,dw)  
       !-
       !- SECHIBA_HISTLEVEL = 2
       !-
       CALL histdef(hist_id, 'subli', 'Sublimation', 'mm/d', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, fluxop(2), dt,dw)
       CALL histdef(hist_id, 'evapnu', 'Bare soil evaporation', 'mm/d', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, fluxop(2), dt,dw)
       CALL histdef(hist_id, 'runoff', 'Surface runoff', 'mm/d', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, fluxop(2), dt,dw)
       CALL histdef(hist_id, 'drainage', 'Deep drainage', 'mm/d', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, fluxop(2), dt,dw)
       CALL histdef(hist_id, 'riversret', 'Return from endorheic rivers', 'mm/d', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, fluxop(2), dt,dw)
       CALL histdef(hist_id, 'hydrographs', 'Hydrographs of gridbox outflow', 'm^3/s', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, fluxop_scinsec(2), dt,dw)
       !
       CALL histdef(hist_id, 'tair', 'Air Temperature', 'K',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, ave(2), dt,dw)
       CALL histdef(hist_id, 'qair', 'Air humidity', 'g/g',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, ave(2), dt,dw)
       CALL histdef(hist_id, 'alb_vis', 'Albedo visible', '-', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, ave(2), dt,dw)
       CALL histdef(hist_id, 'alb_nir', 'Albedo near infrared', '-', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, ave(2), dt,dw)
       CALL histdef(hist_id, 'z0', 'Surface roughness', 'm',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(2), dt,dw)
       CALL histdef(hist_id, 'transpir', 'Transpiration', 'mm/d', &
            & iim,jjm, hori_id, nvm, 1, nvm, vegax_id, 32, fluxop(2), dt,dw)
       CALL histdef(hist_id, 'inter', 'Interception loss', 'mm/d', &
            & iim,jjm, hori_id, nvm, 1, nvm, vegax_id, 32, fluxop(2), dt,dw)
       !-
       !- SECHIBA_HISTLEVEL = 3
       !-
       CALL histdef(hist_id, 'tsol_max', 'Maximum Surface Temperature',&
            & 'C', iim,jjm, hori_id, 1,1,1, -99, 32, tmaxcels(3), dt,dw)
       CALL histdef(hist_id, 'tsol_min', 'Minimum Surface Temperature',&
            & 'C', iim,jjm, hori_id, 1,1,1, -99, 32, tmincels(3), dt,dw)
       CALL histdef(hist_id, 'fluxsens', 'Sensible Heat Flux', 'W/m^2',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, ave(3), dt,dw)
       CALL histdef(hist_id, 'fluxlat', 'Latent Heat Flux', 'W/m^2',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, ave(3), dt,dw)
       CALL histdef(hist_id, 'snow', 'Snow mass', 'kg/m^2', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(3), dt,dw)
       CALL histdef(hist_id, 'snowage', 'Snow age', '?', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(3), dt,dw)
       CALL histdef(hist_id, 'snownobio', 'Snow on other surfaces', 'kg/m^2', &
            & iim,jjm, hori_id, nnobio,1, nnobio, nobioax_id, 32, avescatter(3), dt,dw)
       CALL histdef(hist_id, 'vegetfrac', 'Fraction of vegetation', '-', &
            & iim,jjm, hori_id, nvm, 1, nvm, vegax_id, 32, avescatter(3), dt,dw)
       CALL histdef(hist_id, 'maxvegetfrac', 'Maximum fraction of vegetation', '-', &
            & iim,jjm, hori_id, nvm, 1, nvm, vegax_id, 32, avescatter(3), dt,dw)
       CALL histdef(hist_id, 'nobiofrac', 'Fraction of other surface types', '-', &
            & iim,jjm, hori_id, nnobio, 1, nnobio, nobioax_id, 32, avescatter(3), dt,dw)
       !-
       !- SECHIBA_HISTLEVEL = 4
       !-
       CALL histdef(hist_id, 'gqsb', 'Upper Soil Moisture', '-',  &
            & iim,jjm, hori_id, 1, 1, 1, -99, 32, avescatter(4), dt,dw)
       CALL histdef(hist_id, 'bqsb', 'Lower Soil Moisture', '-',  &
            & iim,jjm, hori_id, 1, 1, 1, -99, 32, avescatter(4), dt,dw)
       CALL histdef(hist_id, 'qsintveg', 'Water on canopy', 'Kg/m^2', &
            & iim,jjm, hori_id, nvm, 1, nvm, vegax_id, 32, avescatter(4), dt,dw)
       CALL histdef(hist_id, 'hdry', 'Dry soil height', 'm',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(4), dt,dw)
       CALL histdef(hist_id, 'evapot', 'Potential evaporation', 'mm/d',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, fluxop(4), dt,dw)
       !-
       !- SECHIBA_HISTLEVEL = 5
       !-
       CALL histdef(hist_id, 'swnet', 'Net solar radiation', 'W/m^2',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, ave(5), dt,dw)
       CALL histdef(hist_id, 'lwdown', 'Absorbed downward longwave radiation', 'W/m^2',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(5), dt,dw)
       CALL histdef(hist_id, 'lwnet', 'Net surface longwave radiation', 'W/m^2',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(5), dt,dw)
       CALL histdef(hist_id, 'lakevol', 'Volume in lake reservoir', 'kg/m^2', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(5), dt,dw)
       CALL histdef(hist_id, 'gpp', 'CO2 Assimilation', 'gC/m2/d', &
            & iim,jjm, hori_id, nvm, 1, nvm, vegax_id, 32, fluxop(5), dt,dw)
       !-
       !- SECHIBA_HISTLEVEL = 6
       !-
       CALL histdef(hist_id, 'ptn', 'Deep ground temperature', 'K', &
            & iim,jjm, hori_id, ngrnd, 1, ngrnd, solax_id, 32, avescatter(6),  dt,dw)
       CALL histdef(hist_id, 'precisol', 'Throughfall', 'mm/d',  &
            & iim,jjm, hori_id, nvm, 1, nvm, vegax_id, 32, fluxop(6), dt,dw)
       CALL histdef(hist_id, 'lai', 'Leaf Area Index', '-', &
            & iim,jjm, hori_id, nvm, 1, nvm, vegax_id, 32, avescatter(6), dt,dw)
       CALL histdef(hist_id, 'rstruct', 'Structural resistance', 's/m', &
            & iim,jjm, hori_id, nvm, 1, nvm, vegax_id, 32, avescatter(6), dt,dw)
       CALL histdef(hist_id, 'roughheight', 'Effective roughness height', 'm',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(6), dt,dw)
       CALL histdef(hist_id, 'temp_pheno', 'Temperature for Pheno', 'K',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(6), dt,dw)
       CALL histdef(hist_id, 'nee', 'Net CO2 Ecossytem Exchange', 'gC/m2/d', &
            & iim,jjm, hori_id, 1, 1, 1, -99, 32, fluxop(6), dt,dw)
       !-
       !- SECHIBA_HISTLEVEL = 7
       !-
       CALL histdef(hist_id, 'fastr', 'Fast flow reservoir', 'kg/m^2', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(7), dt,dw)
       CALL histdef(hist_id, 'slowr', 'Slow flow reservoir', 'kg/m^2', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(7), dt,dw)
       CALL histdef(hist_id, 'streamr', 'Stream flow reservoir', 'kg/m^2', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(7), dt,dw)
       CALL histdef(hist_id, 'irrigation', 'Net irrigation', 'mm/d', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, fluxop(7), dt,dw)
       CALL histdef(hist_id, 'irrigmap', 'Map of irrigated areas', '%', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, once(7), dt,dw)
       CALL histdef(hist_id, 'snownobioage', 'Snow age on other surfaces', 'd', &
            & iim,jjm, hori_id, nnobio,1, nnobio, nobioax_id, 32, avescatter(7), dt,dw)
       !-
       !- SECHIBA_HISTLEVEL = 8
       !-
       CALL histdef(hist_id, 'beta', 'Beta Function', '-',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(8), dt,dw)
       CALL histdef(hist_id, 'raero', 'Aerodynamic resistance', 's/m',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(8), dt,dw)
       CALL histdef(hist_id, 'vbeta1', 'Beta for sublimation', '-',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(8), dt,dw)
       CALL histdef(hist_id, 'vbeta4', 'Beta for bare soil', '-',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(8), dt,dw)
       CALL histdef(hist_id, 'vbetaco2', 'beta for CO2', 'mm/d', &
            & iim,jjm, hori_id, nvm, 1, nvm, vegax_id, 32, avescatter(8), dt,dw)
       CALL histdef(hist_id, 'soiltype', 'Fraction of soil textures', '%', &
            & iim,jjm, hori_id, nbsoil, 1, nbsoil, soltax_id, 32, once(8),  dt,dw)
       CALL histdef(hist_id, 'humrel', 'Soil moisture stress', '-',  &
            & iim,jjm, hori_id, nvm,1,nvm, vegax_id, 32, avescatter(8), dt,dw)
       !-
       !- SECHIBA_HISTLEVEL = 10
       !-
       CALL histdef(hist_id, 'cimean', 'Stomatal CO2 concentation', 'mmole/m2/s', &
            & iim,jjm, hori_id, nvm, 1, nvm, vegax_id, 32, avescatter(10), dt,dw)
       CALL histdef(hist_id, 'vbeta3', 'Beta for Transpiration', 'mm/d', &
            & iim,jjm, hori_id, nvm, 1, nvm, vegax_id, 32, avescatter(10), dt,dw)
       CALL histdef(hist_id, 'rveget', 'Surface resistance', 's/m', &
            & iim,jjm, hori_id, nvm, 1, nvm, vegax_id, 32, avescatter(10), dt,dw)
       CALL histdef(hist_id,'vbeta2','Beta for Interception loss','mm/d', &
            & iim,jjm, hori_id, nvm, 1, nvm, vegax_id, 32, avescatter(10), dt,dw)
       CALL histdef(hist_id, 'qsintmax', 'Maximum Interception capacity', 'Kg/m^2', &
            & iim,jjm, hori_id, nvm, 1, nvm, vegax_id, 32, avescatter(10), dt,dw)
    ELSE 
       !-
       !- This is the ALMA convention output now
       !-
       !- 
       IF ( rectilinear ) THEN
          CALL histbeg(histname, iim, lon_rect, jjm, lat_rect, 1, iim, 1, jjm, &
               &     istp_old, date0, dt, hori_id, hist_id)
       ELSE
          CALL histbeg(histname, iim, lon, jjm, lat, 1, iim, 1, jjm, &
               &     istp_old, date0, dt, hori_id, hist_id)
       ENDIF
       !-
       CALL histvert(hist_id, 'veget', 'Vegetation types', '-', &
            &    nvm,   veg, vegax_id)
       CALL histvert(hist_id, 'solth', 'Soil levels',      'm', &
            &    ngrnd, sol, solax_id)
       CALL histvert(hist_id, 'soiltyp', 'Soil types',      '-', &
            &    nbsoil, soltyp, soltax_id)
       CALL histvert(hist_id, 'nobio', 'Other surface types',      '-', &
            &    nnobio, nobiotyp, nobioax_id)
     !-
     !-  Vegetation
     !-
       CALL histdef(hist_id, 'vegetfrac', 'Fraction of vegetation', '-', &
            & iim,jjm, hori_id, nvm, 1, nvm, vegax_id, 32, avescatter(3), dt,dw)
       CALL histdef(hist_id, 'maxvegetfrac', 'Maximum fraction of vegetation', '-', &
            & iim,jjm, hori_id, nvm, 1, nvm, vegax_id, 32, avescatter(3), dt,dw)
       CALL histdef(hist_id, 'nobiofrac', 'Fraction of other surface types', '-', &
            & iim,jjm, hori_id, nnobio, 1, nnobio, nobioax_id, 32, avescatter(3), dt,dw)
     !- 
     !-  General energy balance
     !-
       CALL histdef(hist_id, 'SWnet', 'Net shortwave radiation', 'W/m^2',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, ave(1), dt,dw)
       CALL histdef(hist_id, 'LWnet', 'Net longwave radiation', 'W/m^2',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(1), dt,dw)
       CALL histdef(hist_id, 'Qh', 'Sensible heat flux', 'W/m^2',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, ave(1), dt,dw)
       CALL histdef(hist_id, 'Qle', 'Latent heat flux', 'W/m^2',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, ave(1), dt,dw)
       CALL histdef(hist_id, 'Qg', 'Ground heat flux', 'W/m^2',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(1), dt,dw)
       CALL histdef(hist_id, 'Qf', 'Energy of fusion', 'W/m^2',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, ave(2), dt,dw)
       CALL histdef(hist_id, 'Qv', 'Energy of sublimation', 'W/m^2',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(1), dt,dw)
       CALL histdef(hist_id, 'DelSurfHeat', 'Change in surface layer heat', 'J/m^2',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, sumscatter(1), dt,dw)
       CALL histdef(hist_id, 'DelColdCont', 'Change in snow surface layer cold content', 'J/m^2',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, sumscatter(1), dt,dw)
    !-
    !- General water balance
    !-
       CALL histdef(hist_id, 'Snowf', 'Snowfall rate', 'kg/m^2/s',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, fluxop_scinsec(1), dt,dw)
       CALL histdef(hist_id, 'Rainf', 'Rainfall rate', 'kg/m^2/s',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, fluxop_scinsec(1), dt,dw)
       CALL histdef(hist_id, 'Evap', 'Total Evapotranspiration', 'kg/m^2/s', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, fluxop_scinsec(1), dt,dw)
       CALL histdef(hist_id, 'Qs', 'Surface runoff', 'kg/m^2/s', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, fluxop_scinsec(1), dt,dw)
       CALL histdef(hist_id, 'Qsb', 'Sub-surface runoff', 'kg/m^2/s', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, fluxop_scinsec(1), dt,dw)
       CALL histdef(hist_id, 'Qsm', 'Snowmelt', 'kg/m^2/s', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, fluxop_scinsec(1), dt,dw)
       CALL histdef(hist_id, 'DelSoilMoist', 'Change in soil moisture', 'kg/m^2',  &
            & iim,jjm, hori_id, 1, 1, 1, -99, 32, sumscatter(1), dt,dw)
       CALL histdef(hist_id, 'DelSWE', 'Change in SWE','kg/m^2',&
            & iim,jjm, hori_id, 1, 1, 1, -99, 32, sumscatter(1), dt,dw)
       CALL histdef(hist_id, 'DelIntercept', 'Change in interception storage', 'kg/m^2',  &
            & iim,jjm, hori_id, 1, 1, 1, -99, 32, sumscatter(1), dt,dw)

    !-
    !- Surface state
    !-
       CALL histdef(hist_id, 'AvgSurfT', 'Average surface temperature', 'K', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, ave(1), dt,dw)
       CALL histdef(hist_id, 'RadT', 'Surface radiative temperature', 'K', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, ave(1), dt,dw)
       CALL histdef(hist_id, 'Albedo', 'Albedo', '-', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(1), dt,dw)
       CALL histdef(hist_id, 'SWE', '3D soil water equivalent','kg/m^2',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(1), dt,dw)
    !!-
    !-  Sub-surface state
    !- 
       CALL histdef(hist_id, 'SoilMoist', '3D average layer soil moisture', 'kg/m^2',  &
            & iim,jjm, hori_id, 1, 1, 1, -99, 32, avescatter(1), dt,dw)
       CALL histdef(hist_id, 'SoilWet', 'Total soil wetness', 'kg/m^2',  &
            & iim,jjm, hori_id, 1, 1, 1, -99, 32, avescatter(1), dt,dw)
       CALL histdef(hist_id, 'SoilTemp', '3D layer average soil temperature', 'K', &
            & iim,jjm, hori_id, ngrnd, 1, ngrnd, solax_id, 32, avescatter(1),  dt,dw)
    !- 
    !-  Evaporation components
    !-
       CALL histdef(hist_id, 'PotEvap', 'Potential evapotranspiration', 'kg/m^2/s', &
            & iim,jjm, hori_id, 1, 1, 1, -99, 32, fluxop_scinsec(1), dt,dw)
       CALL histdef(hist_id, 'ECanop', 'Interception evaporation', 'kg/m^2/s', &
            & iim,jjm, hori_id, nvm, 1, nvm, vegax_id, 32, fluxop_scinsec(1), dt,dw)
       CALL histdef(hist_id, 'TVeg', 'Transpiration', 'kg/m^2/s', &
            & iim,jjm, hori_id, nvm, 1, nvm, vegax_id, 32, fluxop_scinsec(1), dt,dw)
       CALL histdef(hist_id, 'ESoil', 'Bare soil evaporation', 'kg/m^2/s', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, fluxop_scinsec(1), dt,dw)
       CALL histdef(hist_id, 'RootMoist','Root zone soil water', 'kg/m^2',  &
            & iim,jjm, hori_id, 1, 1, 1, -99, 32, avescatter(1), dt,dw)
       CALL histdef(hist_id, 'SubSnow','Snow sublimation','kg/m^2/s', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, fluxop_scinsec(1), dt,dw)
       CALL histdef(hist_id, 'ACond', 'Aerodynamic conductance', 'm/s',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(1), dt,dw)
       !-
    !-
    !-  Cold Season Processes
    !-
       CALL histdef(hist_id, 'SnowFrac', 'Snow cover fraction', '-',  &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(1), dt,dw)
       CALL histdef(hist_id, 'SAlbedo', 'Snow albedo', '-', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(1), dt,dw)
       CALL histdef(hist_id, 'SnowDepth', '3D snow depth', 'm', &
            & iim,jjm, hori_id, 1,1,1, -99, 32, avescatter(1), dt,dw)
    !-
    !-  The carbon budget
    !-
       IF ( control_flags%ok_co2 ) THEN
          CALL histdef(hist_id, 'GPP', 'Net assimilation of carbon by the vegetation', 'gC/m^2/s', &
               & iim,jjm, hori_id, nvm, 1, nvm, vegax_id, 32, fluxop_scinsec(1), dt,dw)
       ENDIF
       IF ( control_flags%ok_stomate ) THEN
          CALL histdef(hist_id, 'NEE', 'Net Ecosystem Exchange', 'gC/m^2/s', &
               & iim,jjm, hori_id, 1, 1, 1, vegax_id, 32, fluxop_scinsec(1), dt,dw)
       ENDIF
    !
    ENDIF
    !-
    CALL histend(hist_id)
    !-
    !=====================================================================
    !- 3.2 STOMATE's history file
    !=====================================================================
    IF ( control_flags%ok_co2 ) THEN
       !-
       ! STOMATE IS ACTIVATED
       !-
       !Config  Key  = STOMATE_OUTPUT_FILE
       !Config  Desc = Name of file in which STOMATE's output is going
       !Config         to be written
       !Config  Def  = stomate_history.nc
       !Config  Help = This file is going to be created by the model
       !Config         and will contain the output from the model.
       !Config         This file is a truly COADS compliant netCDF file.
       !Config         It will be generated by the hist software from
       !Config         the IOIPSL package.
       !-
       stom_histname='stomate_history.nc'
       CALL getin('STOMATE_OUTPUT_FILE', stom_histname)
       WRITE(*,*) 'STOMATE_OUTPUT_FILE', stom_histname
       !-
       !Config  Key  = STOMATE_HIST_DT
       !Config  Desc = STOMATE history time step (d)
       !Config  Def  = 10.
       !Config  Help = Time step of the STOMATE history file
       !-
       hist_days_stom = 10.
       CALL getin('STOMATE_HIST_DT', hist_days_stom)
       hist_dt_stom = NINT( hist_days_stom ) * un_jour
       WRITE(*,*) 'output frequency for STOMATE history file (d): ', &
            hist_dt_stom/un_jour
       !-
       !- initialize
       IF ( rectilinear ) THEN
          CALL histbeg(stom_histname, iim, lon_rect, jjm, lat_rect,  1, iim, 1, jjm, &
               &     istp_old, date0, dt, hori_id, hist_id_stom)
       ELSE
          CALL histbeg(stom_histname, iim, lon, jjm, lat,  1, iim, 1, jjm, &
               &     istp_old, date0, dt, hori_id, hist_id_stom)
       ENDIF
       !- define PFT axis
       hist_PFTaxis = (/ ( REAL(i,stnd), i=1,npft ) /)
       !- declare this axis
       CALL histvert (hist_id_stom, 'PFT', 'Plant functional type', &
            & '-', npft, hist_PFTaxis, hist_PFTaxis_id)
       !- define STOMATE history file
       CALL stom_define_history (hist_id_stom, npft, iim, jjm, &
            & dt, hist_dt_stom, hori_id, hist_PFTaxis_id)
       !- end definition
       CALL histend(hist_id_stom)
       !-
    ENDIF
    RETURN

  END SUBROUTINE intsurf_history
  
SUBROUTINE stom_define_history &
& (hist_id_stom, npft, iim, jjm, dt, &
&  hist_dt, hist_hori_id, hist_PFTaxis_id)
!---------------------------------------------------------------------
!- Tell ioipsl which variables are to be written
!- and on which grid they are defined
!---------------------------------------------------------------------

!-
!- Input
!-
!- File id
   INTEGER(long),INTENT(in) :: hist_id_stom
!- number of PFTs
   INTEGER(long),INTENT(in) :: npft
!- Domain size
   INTEGER(long),INTENT(in) :: iim, jjm
!- Time step of STOMATE (seconds)
   REAL(stnd),INTENT(in)    :: dt
!- Time step of history file (s)
   REAL(stnd),INTENT(in)    :: hist_dt
!- id horizontal grid
   INTEGER(long),INTENT(in) :: hist_hori_id
!- id of PFT axis
   INTEGER(long),INTENT(in) :: hist_PFTaxis_id
!-
!- 1 local
!-
!- maximum history level
   INTEGER(long), PARAMETER  :: max_hist_level = 10
!- output level (between 0 and 10)
!-  ( 0:nothing is written, 10:everything is written)
   INTEGER(long)             :: hist_level
!- Character strings to define operations for histdef
   CHARACTER(LEN=40),DIMENSION(max_hist_level) :: ave
!-
!- Define variables to histdef
!-
   INTEGER(long) :: i,k1,k2,k3,k4,k5
!- Number of variables
   INTEGER(long),PARAMETER  :: n_v = 73
!- Names of variables
   CHARACTER(LEN=20),DIMENSION(n_v) :: v_n = (/ &
 'SPACE_NAT           ','LITTER_STR_AB_NAT   ','LITTER_STR_AB_AGRI  ',&
 'LITTER_MET_AB_NAT   ','LITTER_MET_AB_AGRI  ','LITTER_STR_BE_NAT   ',&
 'LITTER_STR_BE_AGRI  ','LITTER_MET_BE_NAT   ','LITTER_MET_BE_AGRI  ',&
 'DEADLEAF_COVER      ','CARBON_ACTIVE_NAT   ','CARBON_ACTIVE_AGRI  ',&
 'CARBON_SLOW_NAT     ','CARBON_SLOW_AGRI    ','CARBON_PASSIVE_NAT  ',&
 'CARBON_PASSIVE_AGRI ','T2M_MONTH           ','T2M_WEEK            ',&
 'HET_RESP_NAT        ','HET_RESP_AGRI       ','BLACK_CARBON        ',&
 'FIREFRAC_NAT        ','FIREFRAC_AGRI       ','FIREINDEX_NAT       ',&
 'LITTERHUM           ','CO2FLUX             ','CO2_FIRE            ',&
 'CO2_TAKEN           ','LAI                 ','VEGET               ',&
 'VEGET_MAX           ','NPP                 ','GPP                 ',&
 'IND                 ','LEAF_M              ','SAP_M_AB            ',&
 'SAP_M_BE            ','HEART_M_AB          ','HEART_M_BE          ',&
 'ROOT_M              ','FRUIT_M             ','RESERVE_M           ',&
 'LEAF_TURN           ','SAP_AB_TURN         ','ROOT_TURN           ',&
 'FRUIT_TURN          ','LEAF_BM_LITTER      ','SAP_AB_BM_LITTER    ',&
 'SAP_BE_BM_LITTER    ','HEART_AB_BM_LITTER  ','HEART_BE_BM_LITTER  ',&
 'ROOT_BM_LITTER      ','FRUIT_BM_LITTER     ','RESERVE_BM_LITTER   ',&
 'MAINT_RESP          ','GROWTH_RESP         ','AGE                 ',&
 'HEIGHT              ','MOISTRESS           ','VCMAX               ',&
 'LEAF_AGE            ','MORTALITY           ','FIREDEATH           ',&
 'IND_ESTAB           ','LIGHT_DEATH         ','BM_ALLOC_LEAF       ',&
 'BM_ALLOC_SAP_AB     ','BM_ALLOC_SAP_BE     ','BM_ALLOC_ROOT       ',&
 'BM_ALLOC_FRUIT      ','BM_ALLOC_RES        ','HERBIVORES          ',&
 'TURNOVER_TIME       '/)

!---------------------------------
!- Descriptions of variables
   CHARACTER(LEN=50),DIMENSION(n_v) :: v_d = (/ &
 'fraction of total space that is natural           ',&
 'structural litter above nat. ground               ',&
 'structural litter above agric. ground             ',&
 'metabolic litter above nat. ground                ',&
 'metabolic litter above agric. ground              ',&
 'structural litter below nat. ground               ',&
 'structural litter below agric. ground             ',&
 'metabolic litter below nat. ground                ',&
 'metabolic litter below agric. ground              ',&
 'fraction of soil covered by dead leaves           ',&
 'active soil carbon in nat. ground                 ',&
 'active soil carbon in agric. ground               ',&
 'slow soil carbon in nat. ground                   ',&
 'slow soil carbon in agric. ground                 ',&
 'passive soil carbon in nat. ground                ',&
 'passive soil carbon in agric. ground              ',&
 'Monthly 2 m temperature                           ',&
 'Weekly 2 m temperature                            ',&
 'heterotr. resp. from nat. ground                  ',&
 'heterotr. resp. from agric. ground                ',&
 'black carbon on average total ground              ',&
 'Fire fraction on natural ground                   ',&
 'Fire fraction on agricultural ground              ',&
 'Fire index on natural ground                      ',&
 'Litter humidity                                   ',&
 'Total output CO2 flux                             ',&
 'Output CO2 flux from fire                         ',&
 'CO2 taken from atmosphere for initiate growth     ',&
 'Leaf Area Index                                   ',&
 'Vegetation fraction                               ',&
 'Maximum vegetation fraction (LAI -> infinity)     ',&
 'Net primary productivity                          ',&
 'Gross primary productivity                        ',&
 'Density of individuals                            ',&
 'Leaf mass                                         ',&
 'Sap mass above ground                             ',&
 'Sap mass below ground                             ',&
 'Heartwood mass above ground                       ',&
 'Heartwood mass below ground                       ',&
 'Root mass                                         ',&
 'Fruit mass                                        ',&
 'Carbohydrate reserve mass                         ',&
 'Leaf turnover                                     ',&
 'Sap turnover above                                ',&
 'Root turnover                                     ',&
 'Fruit turnover                                    ',&
 'Leaf death                                        ',&
 'Sap death above ground                            ',&
 'Sap death below ground                            ',&
 'Heartwood death above ground                      ',&
 'Heartwood death below ground                      ',&
 'Root death                                        ',&
 'Fruit death                                       ',&
 'Carbohydrate reserve death                        ',&
 'Maintenance respiration                           ',&
 'Growth respiration                                ',&
 'age                                               ',&
 'height                                            ',&
 'weekly moisture stress                            ',&
 'Maximum rate of carboxylation                     ',&
 'leaf age                                          ',&
 'Fraction of trees that dies (gap)                 ',&
 'Fraction of plants killed by fire                 ',&
 'Density of newly established saplings             ',&
 'Fraction of plants that dies (light competition)  ',&
 'biomass allocated to leaves                       ',&
 'biomass allocated to sapwood above ground         ',&
 'biomass allocated to sapwood below ground         ',&
 'biomass allocated to roots                        ',&
 'biomass allocated to fruits                       ',&
 'biomass allocated to carbohydrate reserve         ',&
 'time constant of herbivore activity               ',&
 'turnover time for grass leaves                    ' /) 
!- Units of variables
   CHARACTER(LEN=20),DIMENSION(n_v) :: v_u = (/ &
 '-                   ','gC/m^2 (n/a)        ','gC/m^2 (n/a)        ',&
 'gC/m^2 (n/a)        ','gC/m^2 (n/a)        ','gC/m^2 (n/a)        ',&
 'gC/m^2 (n/a)        ','gC/m^2 (n/a)        ','gC/m^2 (n/a)        ',&
 '-                   ','gC/m^2 (n/a)        ','gC/m^2 (n/a)        ',&
 'gC/m^2 (n/a)        ','gC/m^2 (n/a)        ','gC/m^2 (n/a)        ',&
 'gC/m^2 (n/a)        ','K                   ','K                   ',&
 'gC/m^2 tot/day      ','gC/m^2 tot/day      ','gC/m^2 tot          ',&
 '1/day               ','1/day               ','-                   ',&
 '-                   ','gC/day/(m^2 tot)    ','gC/day/(m^2 (n/a))  ',&
 'gC/day/(m^2 (n/a))  ','-                   ','-                   ',&
 '-                   ','gC/day/(m^2 tot)    ','gC/day/(m^2 (n/a))  ',&
 '1/(m^2 (n/a))       ','gC/m^2 (n/a)        ','gC/m^2 (n/a)        ',&
 'gC/m^2 (n/a)        ','gC/m^2 (n/a)        ','gC/m^2 (n/a)        ',&
 'gC/m^2 (n/a)        ','gC/m^2 (n/a)        ','gC/m^2 (n/a)        ',&
 'gC/m^2 (n/a)/day    ','gC/m^2 (n/a)/day    ','gC/m^2 (n/a)/day    ',&
 'gC/m^2 (n/a)/day    ','gC/m^2 (n/a)/day    ','gC/m^2 (n/a)/day    ',&
 'gC/m^2 (n/a)/day    ','gC/m^2 (n/a)/day    ','gC/m^2 (n/a)/day    ',&
 'gC/m^2 (n/a)/day    ','gC/m^2 (n/a)/day    ','gC/m^2 (n/a)/day    ',&
 'gC/m^2 tot/day      ','gC/m^2 tot/day      ','years               ',&
 'm                   ','-                   ','-                   ',&
 'days                ','1/day               ','1/day               ',&
 '1/day               ','1/day               ','gC/m**2 nat/agri/dt ',&
 'gC/m**2 nat/agri/dt ','gC/m**2 nat/agri/dt ','gC/m**2 nat/agri/dt ',&
 'gC/m**2 nat/agri/dt ','gC/m**2 nat/agri/dt ','days                ',&
 'days                '/)
!- Dimensions of variables
   INTEGER(long),DIMENSION(n_v) :: v_t = (/ (2,i=1,28), (3,i=29,n_v) /)
!- Level of operations for variables
   INTEGER(long),DIMENSION(n_v) :: v_l = (/ &
    1,  5,  5,  5,  5,  5,  5,  5,  5,  5, &
    5,  5,  5,  5,  5,  5,  1,  1,  3,  3, &
   10,  1,  1, 10,  5,  1,  1,  1,  1,  1, &
    1,  1,  1,  3,  2,  2,  2,  2,  2,  2, &
    2,  2,  4,  4,  4,  4,  4,  4,  4,  4, & 
    4,  4,  4,  4,  2,  2,  7,  7,  3,  6, &
    6,  6,  6,  6,  6,  5,  5,  5,  5,  5, &
    5,  5,  5       /)
!---------------------------------------------------------------------
!=====================================================================
!- 1 history level
!=====================================================================
!- 1.1 define history levelx
!=====================================================================
   !Config  Key  = STOMATE_HISTLEVEL
   !Config  Desc = STOMATE history output level (0..10)
   !Config  Def  = 10
   !Config  Help = 0: nothing is written; 10: everything is written
!-
   hist_level = 10
   CALL getin('STOMATE_HISTLEVEL', hist_level)
!-
   WRITE(*,*) 'STOMATE history level: ',hist_level
   IF ( (hist_level > max_hist_level).OR.(hist_level < 0) ) THEN
     STOP 'This history level is not allowed'
   ENDIF
!=====================================================================
!- 1.2 define operations according to output level
!=====================================================================
   ave(1:hist_level) =  'ave(scatter(X))'
   ave(hist_level+1:max_hist_level) =  'never          '
!=====================================================================
!- 2 surface fields (2d)
!- 3 PFT: 3rd dimension
!=====================================================================
   k2 = 1; k5 = 32;
   DO i=1,n_v
     IF      (v_t(i) == 2) THEN
       k1 = 1; k3 = 1; k4 = -99;
     ELSE IF (v_t(i) == 3) THEN
       k1 = npft; k3 = npft; k4 = hist_PFTaxis_id;
     ELSE
       CYCLE
     ENDIF
     CALL histdef (hist_id_stom, &
     & TRIM(v_n(i)),TRIM(v_d(i)),TRIM(v_u(i)), iim,jjm, &
     & hist_hori_id, k1,k2,k3,k4,k5, ave(v_l(i)), dt, hist_dt)
   ENDDO
   CALL histdef(hist_id_stom, 'RESOLUTION_X', 'E-W resolution', 'm', &
   & iim,jjm, hist_hori_id, 1,1,1, -99, 32, 'once(scatter(X))', dt, hist_dt)
   CALL histdef(hist_id_stom, 'RESOLUTION_Y', 'N-S resolution', 'm', &
   & iim,jjm, hist_hori_id, 1,1,1, -99, 32, 'once(scatter(X))', dt, hist_dt)

!---------------------------------
END SUBROUTINE stom_define_history
!
!
!
SUBROUTINE grid_stuff (npts, iim, jjm, grid_lon, grid_lat, kindex, &
                         neighbours, resolution)

  !
  ! 0 interface
  !
  
  !
  ! 0.1 input  !
  
  ! Domain size
  INTEGER(short), INTENT(in)                                  :: npts
  ! Size of cartesian grid
  INTEGER(short), INTENT(in)                                  :: iim, jjm
  ! Longitudes on cartesian grid
  REAL(stnd), DIMENSION(iim,jjm), INTENT(in)           :: grid_lon
  ! Latitudes on cartesian grid
  REAL(stnd), DIMENSION(iim,jjm), INTENT(in)           :: grid_lat
  ! Indices of STOMATE points on cartesian grid
  INTEGER(short), DIMENSION(npts), INTENT(in)                 :: kindex

  !
  ! 0.2 output
  !
  
  ! indices of the 4 neighbours of each grid point (1=N, 2=E, 3=S, 4=W)
  !   a zero or negative index means that this neighbour is not a land point
  INTEGER(short), DIMENSION(npts,8), INTENT(out)              :: neighbours
  ! resolution at each grid point in m (1=E-W, 2=N-S)
  REAL(stnd), DIMENSION(npts,2), INTENT(out)           :: resolution
  
  !
  ! 0.3 local
  !
  
  ! default resolution (m)
  REAL(stnd)                                                 :: default_resolution
  ! which STOMATE point corresponds to the given point on the cartesian grid
  INTEGER, DIMENSION(iim,jjm)                          :: correspondance
  ! cosine of the latitude
  REAL(stnd)                                                 :: coslat
  ! number of points where default resolution is used
  INTEGER(short)                                              :: ndefault_lon, ndefault_lat
  ! Indices
  INTEGER(short)                                              :: i,ip,jp, imm1, imp1, imm1l, imp1l, ii
  !
  LOGICAL                                              :: global = .FALSE.
  !
  CHARACTER(LEN=2), DIMENSION(2)                       :: grid_dir
  ! Rose gives the geographical direction for the various index increments
  ! The following corespondences exist
  !                       WE&NS  WE&SN and so on !
  ! rose(1) = i+0 & j-1    NN     SS
  ! rose(2) = i+1 & j-1    NE     SE
  ! rose(3) = i+1 & j+0    EE     EE
  ! rose(4) = i+1 & j+1    SE     NE
  ! rose(5) = i+0 & j+1    SS     NN
  ! rose(6) = i-1 & j+1    SW     NW
  ! rose(7) = i-1 & j+0    WW     WW
  ! rose(8) = i-1 & j-1    NW     SW
  INTEGER(short), DIMENSION(8)                                :: rose
  !
  REAL(stnd), PARAMETER                          :: R_Earth = 6378000.
  INTEGER(short), SAVE                            :: bavard=1
  REAL(stnd), PARAMETER                          :: min_stomate = 1.E-8
  REAL(stnd), SAVE                               :: pi
  
  ! =========================================================================
  
  pi = 4. * ATAN(1.)
  
  IF ( bavard .GE. 4 ) WRITE(*,*) 'Entering grid_stuff'
  
  ! default resolution
  default_resolution = 250000.
  IF ( bavard .GT. 1 ) WRITE(*,*) 'grid stuff: default resolution (m): ',default_resolution
  !
  ! Check if we have a global map or not.
  ! There is little change that if iim <=2 and jjm <= 2 that we have global grid.
  ! Furthermore using the second line allows to avoid pole problems for global grids
  IF (iim <= 2 .OR. jjm <= 2) THEN
     global = .FALSE.
  ELSE
     ! We assume here that the longitude is in increasing order and in degrees.
     IF ( grid_lon(iim,2)-grid_lon(1,2) >= 360. - (grid_lon(2,2)-grid_lon(1,2)) ) THEN
        global = .TRUE.
     ENDIF
  ENDIF
  !
  ! Get the direction of the grid
  !
  IF ( iim > 1 ) THEN
     IF ( grid_lon(1,1) <= grid_lon(2,1) ) THEN
        grid_dir(1) = 'WE'
     ELSE
        grid_dir(1) = 'EW'
     ENDIF
  ELSE
     grid_dir(1) = 'WE'
  ENDIF
  !
  IF ( jjm > 1 ) THEN
     IF ( grid_lat(1,1) >= grid_lat(1,2) ) THEN
        grid_dir(2) = 'NS'
     ELSE
        grid_dir(2) = 'SN'
     ENDIF
  ELSE
     grid_dir(2) = 'NS'
  ENDIF
  !
  !! WRITE(*,*) 'Longitude direction :', grid_dir(1)
  !! WRITE(*,*) 'Latitude  direction :', grid_dir(2)
  !
  ndefault_lon = 0
  ndefault_lat = 0
  ! initialize output
  neighbours(:,:) = -1
  resolution(:,:) = 0.
  
  correspondance(:,:) = -1
  
  DO i = 1, npts
     
     !
     ! 1 find numbers of the latitude and longitude of each point
     !
     
     ! index of latitude
     jp = INT( (kindex(i)-1) /iim ) + 1
     
     ! index of longitude
     ip = kindex(i) - ( jp-1 ) * iim
     !
     !correspondance(ip,jp) = kindex(i)
     !
     correspondance(ip,jp) = i
  ENDDO
  
  !
  ! Get the "wind rose" for the various orientation of the grid
  !
  IF ( grid_dir(1) .EQ. 'WE' .AND.  grid_dir(2) .EQ. 'NS' ) THEN
     rose(1) = 1
     rose(2) = 2
     rose(3) = 3
     rose(4) = 4
     rose(5) = 5
     rose(6) = 6
     rose(7) = 7
     rose(8) = 8
  ELSE IF ( grid_dir(1) .EQ. 'EW' .AND.  grid_dir(2) .EQ. 'NS' ) THEN
     rose(1) = 1
     rose(2) = 8
     rose(3) = 7
     rose(4) = 6
     rose(5) = 5
     rose(6) = 4
     rose(7) = 3
     rose(8) = 2
  ELSE IF ( grid_dir(1) .EQ. 'WE' .AND.  grid_dir(2) .EQ. 'SN' ) THEN
     rose(1) = 5
     rose(2) = 4
     rose(3) = 3
     rose(4) = 2
     rose(5) = 1
     rose(6) = 8
     rose(7) = 7
     rose(8) = 6
  ELSE IF ( grid_dir(1) .EQ. 'EW' .AND.  grid_dir(2) .EQ. 'SN' ) THEN
     rose(1) = 5
     rose(2) = 6
     rose(3) = 7
     rose(4) = 8
     rose(5) = 1
     rose(6) = 2
     rose(7) = 3
     rose(8) = 4
  ELSE
     WRITE(*,*) 'We can not be here'
     STOP 'grid_stuff'
  ENDIF

  DO i = 1, npts
     
     ! index of latitude
     jp = INT( (kindex(i)-1) /iim ) + 1
     
     ! index of longitude
     ip = kindex(i) - ( jp-1 ) * iim
     
     !
     ! 2 resolution
     !
     
     !
     ! 2.1 longitude
     !
     
     ! prevent infinite resolution at the pole
       coslat = MAX( COS( grid_lat(ip,jp) * pi/180. ), 0.001 )     
     IF ( iim .GT. 1 ) THEN
        
        IF ( ip .EQ. 1 ) THEN
           resolution(i,1) = ABS( grid_lon(ip+1,jp) - grid_lon(ip,jp) ) * &
                pi/180. * R_Earth * coslat
        ELSEIF ( ip .EQ. iim ) THEN
           resolution(i,1) = ABS( grid_lon(ip,jp) - grid_lon(ip-1,jp) ) * &
                pi/180. * R_Earth * coslat
        ELSE
           resolution(i,1) = ABS( grid_lon(ip+1,jp) - grid_lon(ip-1,jp) ) / 2. * &
                pi/180. * R_Earth * coslat
        ENDIF
        
     ELSE
        
        resolution(i,1) = default_resolution
        
        ndefault_lon = ndefault_lon + 1
        
     ENDIF
     
     !
     ! 2.2 latitude
     !
     
     IF ( jjm .GT. 1 ) THEN
        
        IF ( jp .EQ. 1 ) THEN
           resolution(i,2) = ABS( grid_lat(ip,jp) - grid_lat(ip,jp+1) ) * pi/180. * R_Earth
        ELSEIF ( jp .EQ. jjm ) THEN
           resolution(i,2) = ABS( grid_lat(ip,jp-1) - grid_lat(ip,jp) ) * pi/180. * R_Earth
        ELSE
           resolution(i,2) = ABS( grid_lat(ip,jp-1) - grid_lat(ip,jp+1) ) / 2. * pi/180. * R_Earth
        ENDIF
        
     ELSE
        
        resolution(i,2) = default_resolution
        
        ndefault_lat = ndefault_lat + 1
        
     ENDIF
     
     !
     ! 3 find neighbours 
     !     
     imm1 = 0
     IF ( ip .GT. 1 ) THEN
        imm1 = ip - 1
     ELSEIF ( global ) THEN
        imm1 = iim
     ENDIF
     
     imp1 = 0
     IF ( ip .LT. iim ) THEN
        imp1 = ip + 1
     ELSEIF ( global ) THEN
        imp1 = 1
     ENDIF
     !
     ! East and West
     !
     IF ( imp1 > 0 ) THEN
        neighbours(i,rose(3)) = correspondance(imp1,jp)
     ELSE
        neighbours(i,rose(3)) = -1
     ENDIF
     IF ( imm1 > 0 ) THEN
        neighbours(i,rose(7)) = correspondance(imm1,jp)
     ELSE
        neighbours(i,rose(7)) = -1
     ENDIF
     !
     ! North
     !
     IF ( jp .GT. 1 ) THEN

        neighbours(i,rose(1)) = correspondance(ip,jp-1)
        IF ( imp1 > 0 ) THEN
           neighbours(i,rose(2)) = correspondance(imp1,jp-1)
        ELSE
           neighbours(i,rose(2)) = -1
        ENDIF
        IF ( imm1 > 0 ) THEN
           neighbours(i,rose(8)) = correspondance(imm1,jp-1)
        ELSE
           neighbours(i,rose(8)) = -1
        ENDIF

     ELSE
        IF ( global ) THEN
           
           ! special treatment for the pole if we are really in a 2d grid
           
           IF ( ( iim .GT. 1 ) .AND. ( jjm .GT. 1 ) ) THEN
              !
              ii = MOD(ip+iim/2-1,iim)+1
              imm1l = ii - 1
              IF ( imm1l .LT. 1 ) imm1l = iim
              imp1l = ii + 1
              IF ( imp1l .GT. iim ) imp1l = 1
              !
              IF ( ABS( ( grid_lat(ip,jp) ) - 90. ) .LT. min_stomate ) THEN
                 ! the grid point sits exactly on the pole. The neighbour is situated
                 ! at a lower latitude.
                 neighbours(i,rose(1)) = correspondance( ii, jp+1 )
                 neighbours(i,rose(2)) = correspondance( imm1l, jp+1 )
                 neighbours(i,rose(8)) = correspondance( imp1l, jp+1 )
              ELSE
                 ! look across the North Pole
                 neighbours(i,rose(1)) = correspondance( ii, jp )
                 neighbours(i,rose(2)) = correspondance( imm1l, jp )
                 neighbours(i,rose(8)) = correspondance( imp1l, jp )
              ENDIF
           ENDIF
           
        ELSE
           
           neighbours(i,rose(1)) = -1
           neighbours(i,rose(2)) = -1
           neighbours(i,rose(8)) = -1

        ENDIF

     ENDIF
     
     ! South
     IF ( jp .LT. jjm ) THEN
        
        neighbours(i,rose(5)) = correspondance(ip,jp+1)
        IF ( imp1 > 0 ) THEN
           neighbours(i,rose(4)) = correspondance(imp1,jp+1)
        ELSE
           neighbours(i,rose(4)) = -1
        ENDIF
        IF ( imm1 > 0 ) THEN
           neighbours(i,rose(6)) = correspondance(imm1,jp+1)
        ELSE
           neighbours(i,rose(6)) = -1
        ENDIF

     ELSE
        
        IF ( global ) THEN
           
           ! special treatment for the pole if we are really in a 2d grid
           
           IF ( ( iim .GT. 1 ) .AND. ( jjm .GT. 1 ) ) THEN
              !
              ii = MOD(ip+iim/2-1,iim)+1
              imm1l = ii - 1
              IF ( imm1l .LT. 1 ) imm1l = iim
              imp1l = ii + 1
              IF ( imp1l .GT. iim ) imp1l = 1
              !
              IF ( ( ABS( grid_lat(ip,jp) ) - 90. ) .LT. min_stomate ) THEN
                 ! the grid point sits exactly on the pole. The neighbour is situated
                 ! at a lower latitude.
                 neighbours(i,rose(5)) = correspondance( ii, jp-1 )
                 neighbours(i,rose(4)) = correspondance( imm1l, jp-1 )
                 neighbours(i,rose(6)) = correspondance( imp1l, jp-1 )
              ELSE
                 ! look across the South Pole
                 neighbours(i,rose(5)) = correspondance( ii, jp )
                 neighbours(i,rose(4)) = correspondance( imm1l, jp )
                 neighbours(i,rose(6)) = correspondance( imp1l, jp )
              ENDIF
           ENDIF

        ELSE

           neighbours(i,rose(5)) = -1
           neighbours(i,rose(4)) = -1
           neighbours(i,rose(6)) = -1
           
        ENDIF
     ENDIF
     
    ENDDO
    
    IF ( bavard .GT. 1 ) THEN
       WRITE(*,*) '  > Total number of points: ',npts
       WRITE(*,*) '  > Using default zonal resolution at',ndefault_lon,' points.'
       WRITE(*,*) '  > Using default meridional resolution at',ndefault_lat,' points.'
    ENDIF
    
    IF ( bavard .GT. 1 ) WRITE(*,*) 'Leaving grid_stuff'
    
  END SUBROUTINE grid_stuff
  !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !
  SUBROUTINE intersurf_initwatch(iim, jjm, igmax, date0, itau, dt, kindex, lon, lat, zlev, contfrac)
    !
    IMPLICIT NONE
    !
    ! This routine will allow to set up forcing files for ORCHIDEE. The idea is that
    ! during a coupled simulation one write's out all the forcing so that ORCHIDEE can
    ! can be re-run (to equilibrium or for sensitivity) afterwards.
    !
    ! INPUT
    INTEGER(long)   :: iim, jjm, igmax
    REAL(stnd)      :: date0, dt
    INTEGER(long)   :: itau, kindex(igmax)
    REAL(stnd)      :: lon(iim,jjm), lat(iim,jjm), zlev
    REAL(stnd)      :: contfrac(igmax)
    !
    ! OUTPUT
    !
    !
    ! LOCAL
    !
    INTEGER(long)   :: iret, nlonid1, nlatid1, nlevid1, fid, nlandid1, tdimid1
    INTEGER(long)   :: dims(3), ndim
    INTEGER(long)   :: nlonid, nlatid, nlevid, nlandid, varid, contid
    REAL(stnd)      :: lon_min, lon_max, lat_min, lat_max, lev_min, lev_max
    INTEGER(long)   :: yy, dd, hh, mm, leng, i, j, ig
    REAL(stnd)      :: ss
    REAL(stnd),ALLOCATABLE :: tmpdata(:,:)
    CHARACTER(LEN=3)  :: cal(12)
    CHARACTER(LEN=10) :: today, att, axx
    CHARACTER(LEN=30) :: str30
    CHARACTER(LEN=70) :: str70, var, unit, titre, assoc
    CHARACTER(LEN=80) :: stamp, lon_name, lat_name, time_name
    !
    !
    INCLUDE "netcdf.inc"
    !
    cal(1) = 'JAN'
    cal(2) = 'FEB'
    cal(3) = 'MAR'
    cal(4) = 'APR'
    cal(5) = 'MAY'
    cal(6) = 'JUN'
    cal(7) = 'JUL'
    cal(8) = 'AUG'
    cal(9) = 'SEP'
    cal(10) = 'OCT'
    cal(11) = 'NOV'
    cal(12) = 'DEC'
    !
    watchoffset = 0
    !
    iret = NF_CREATE(watchout_file, NF_CLOBBER, fid)
    !
    !   Dimensions
    !
    iret = NF_DEF_DIM(fid, 'x', iim, nlonid1)
    iret = NF_DEF_DIM(fid, 'y', jjm, nlatid1)
    iret = NF_DEF_DIM(fid, 'z', 1, nlevid1)
    !
    iret = NF_DEF_DIM(fid, 'land', igmax, nlandid1)
    iret = NF_DEF_DIM(fid, 'tstep', NF_UNLIMITED, tdimid1)
    !
    !   Coordinate  VARIABLES
    !
    ndim=2
    dims(1) = nlonid1
    dims(2) = nlatid1
    !
    lon_name = 'nav_lon'
    iret = NF_DEF_VAR(fid, lon_name, NF_FLOAT, ndim, dims, nlonid)
    iret = NF_PUT_ATT_TEXT(fid, nlonid, 'units', 12, "degrees_east") 
    !
    lon_min = -180.
    lon_max = 180.
    !
    iret = NF_PUT_ATT_REAL(fid, nlonid, 'valid_min', NF_FLOAT, 1, lon_min)
    iret = NF_PUT_ATT_REAL(fid, nlonid, 'valid_max', NF_FLOAT, 1, lon_max)
    !
    iret = NF_PUT_ATT_TEXT(fid, nlonid, 'long_name', 9, "Longitude")
    !
    lat_name = 'nav_lat'
    iret = NF_DEF_VAR(fid, lat_name, NF_FLOAT, ndim, dims, nlatid)
    iret = NF_PUT_ATT_TEXT(fid, nlatid, 'units', 13, "degrees_north")
    !
    lat_max = 90.
    lat_min = -90.
    !
    iret = NF_PUT_ATT_REAL(fid, nlatid, 'valid_min', NF_FLOAT, 1, lat_min)
    iret = NF_PUT_ATT_REAL(fid, nlatid, 'valid_max', NF_FLOAT, 1, lat_max)
    iret = NF_PUT_ATT_TEXT(fid, nlatid, 'long_name', 8, "Latitude")
    !
    ndim=1
    dims(1) = nlevid1 
    !
    lat_name = 'level'
    iret = NF_DEF_VAR(fid, lat_name, NF_FLOAT, ndim, dims, nlevid)
    iret = NF_PUT_ATT_TEXT(fid, nlevid, 'units', 1, "m")
    !
    lev_max = 2.
    lev_min = 2.
    !
    iret = NF_PUT_ATT_REAL(fid, nlevid, 'valid_min', NF_FLOAT, 1, lev_min)
    iret = NF_PUT_ATT_REAL(fid, nlevid, 'valid_max', NF_FLOAT, 1, lev_max)
    iret = NF_PUT_ATT_TEXT(fid, nlevid, 'long_name', 15, "Vertical levels")
    !
    !
    ndim=1
    dims(1) =  nlandid1
    !
    lon_name = 'land'
    iret = NF_DEF_VAR(fid, lon_name, NF_INT, ndim, dims, nlandid)
    iret = NF_PUT_ATT_TEXT(fid, nlandid, 'compress', 3, "y x") 
    !
    !   Time in real days !
    !
    time_name = 'time'
    ndim = 1
    dims(1)=tdimid1
    iret = NF_DEF_VAR(fid, time_name, NF_FLOAT, ndim, dims, time_id)
    !
    ! Compute an itau offset so that we can relate the itau of the model
    ! to the position in the file
    !
    watchoffset = itau
    !
    CALL ju2ymds(date0, yy, mm, dd, ss)
    hh = INT(ss/3600.)
    ss = ss - hh*3600.
    mm = INT(ss/60.) + 1
    ss = ss - (mm-1)*60.
    WRITE(str70,7000) yy, mm, dd, hh, mm, INT(ss)
    !
    leng = LEN_TRIM(str70)
    iret = NF_PUT_ATT_TEXT(fid, time_id, 'units', leng, str70)
    !
    CALL ioget_calendar(str30)
    leng = LEN_TRIM(str30)
    iret = NF_PUT_ATT_TEXT(fid, time_id, 'calendar', leng, str30)
    !
    iret = NF_PUT_ATT_TEXT(fid, time_id, 'title', 4, 'Time')
    !
    iret = NF_PUT_ATT_TEXT(fid, time_id, 'long_name', 9, 'Time axis')
    !
    WRITE(str70,7001) yy, cal(mm), dd, hh, mm, INT(ss)
    leng = LEN_TRIM(str70)
    iret = NF_PUT_ATT_TEXT(fid, time_id, 'time_origin', leng, str70)
    !
    !   Time steps
    !
    time_name = 'timestp'
    ndim=1
    dims(1)=tdimid1
    iret = NF_DEF_VAR(fid, time_name, NF_INT, ndim, dims, timestp_id)
    !
    WRITE(str70,7002) yy, mm, dd, hh, mm, INT(ss)
    leng = LEN_TRIM(str70)
    iret = NF_PUT_ATT_TEXT(fid, timestp_id, 'units', leng, str70)
    !
    iret = NF_PUT_ATT_TEXT(fid, timestp_id, 'title', 10, 'Time steps')
    !
    iret = NF_PUT_ATT_REAL(fid, timestp_id, 'tstep_sec', NF_FLOAT, 1, dt)
    !
    iret = NF_PUT_ATT_TEXT(fid, timestp_id, 'long_name', 14, 'Time step axis')
    !
    WRITE(str70,7001) yy, cal(mm), dd, hh, mm, INT(ss)
    leng = LEN_TRIM(str70)
    iret = NF_PUT_ATT_TEXT(fid, timestp_id, 'time_origin', leng, str70)
    !
7000 FORMAT('seconds since ', I4.4,'-',I2.2,'-',I2.2,' ',I2.2,':',I2.2,':',I2.2)
7001 FORMAT(' ', I4.4,'-',A3,'-',I2.2,' ',I2.2,':',I2.2,':',I2.2)
7002 FORMAT('timesteps since ', I4.4,'-',I2.2,'-',I2.2,' ',I2.2,':',I2.2,':',I2.2)
    !

    ndim=2
    dims(1) = nlandid1
    dims(2) = tdimid1

    assoc = 'time (nav_lat nav_lon)'
    axx='TYX'
    !
    var = 'SWdown'
    unit = 'W/m^2'
    titre = 'Surface incident shortwave radiation'
    iret = NF_DEF_VAR(fid,var, NF_FLOAT, ndim, dims, varid)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'axis', LEN_TRIM(axx),axx)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'units', LEN_TRIM(unit), unit) 
    iret = NF_PUT_ATT_TEXT(fid, varid, 'long_name', LEN_TRIM(titre), titre)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'associate', LEN_TRIM(assoc), assoc)
    iret = NF_PUT_ATT_REAL(fid, varid, 'missing_value', NF_FLOAT, 1, undef_sechiba)
    soldownid = varid
    !
    var = 'Rainf'
    unit = 'Kg/m^2s'
    titre = 'Rainfall rate'
    iret = NF_DEF_VAR(fid,var, NF_FLOAT, ndim, dims, varid)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'axis', LEN_TRIM(axx),axx)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'units', LEN_TRIM(unit), unit) 
    iret = NF_PUT_ATT_TEXT(fid, varid, 'long_name', LEN_TRIM(titre), titre)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'associate', LEN_TRIM(assoc), assoc)
    iret = NF_PUT_ATT_REAL(fid, varid, 'missing_value', NF_FLOAT, 1, undef_sechiba)
    rainfid = varid
    !
    var = 'Snowf'
    unit = 'Kg/m^2s'
    titre = 'Snowfall rate'
    iret = NF_DEF_VAR(fid,var, NF_FLOAT, ndim, dims, varid)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'axis', LEN_TRIM(axx),axx)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'units', LEN_TRIM(unit), unit) 
    iret = NF_PUT_ATT_TEXT(fid, varid, 'long_name', LEN_TRIM(titre), titre)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'associate', LEN_TRIM(assoc), assoc)
    iret = NF_PUT_ATT_REAL(fid, varid, 'missing_value', NF_FLOAT, 1, undef_sechiba)
    snowfid = varid
    !
    var = 'LWdown'
    unit = 'W/m^2'
    titre = 'Surface incident longwave radiation'
    iret = NF_DEF_VAR(fid,var, NF_FLOAT, ndim, dims, varid)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'axis', LEN_TRIM(axx),axx)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'units', LEN_TRIM(unit), unit) 
    iret = NF_PUT_ATT_TEXT(fid, varid, 'long_name', LEN_TRIM(titre), titre)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'associate', LEN_TRIM(assoc), assoc)
    iret = NF_PUT_ATT_REAL(fid, varid, 'missing_value', NF_FLOAT, 1, undef_sechiba)
    lwradid = varid
    !
    var = 'PSurf'
    unit = 'Pa'
    titre = 'Surface pressure'
    iret = NF_DEF_VAR(fid,var, NF_FLOAT, ndim, dims, varid)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'axis', LEN_TRIM(axx),axx)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'units', LEN_TRIM(unit), unit) 
    iret = NF_PUT_ATT_TEXT(fid, varid, 'long_name', LEN_TRIM(titre), titre)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'associate', LEN_TRIM(assoc), assoc)
    iret = NF_PUT_ATT_REAL(fid, varid, 'missing_value', NF_FLOAT, 1, undef_sechiba)
    psolid = varid
    !
    !
    !  3D Variables to be written
    !
    ndim=3
    dims(1) = nlandid1
    dims(2) = nlevid1
    dims(3) = tdimid1

    assoc = 'time level (nav_lat nav_lon)'
    axx='TZYX'
    !
    var = 'Tair'
    unit = 'K'
    titre = 'Near surface air temperature'
    iret = NF_DEF_VAR(fid,var, NF_FLOAT, ndim, dims, varid)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'axis', LEN_TRIM(axx),axx)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'units', LEN_TRIM(unit), unit) 
    iret = NF_PUT_ATT_TEXT(fid, varid, 'long_name', LEN_TRIM(titre), titre)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'associate', LEN_TRIM(assoc), assoc)
    iret = NF_PUT_ATT_REAL(fid, varid, 'missing_value', NF_FLOAT, 1, undef_sechiba)
    tairid = varid
    !
    var = 'Qair'
    unit = 'Kg/Kg'
    titre = 'Near surface specific humidity'
    iret = NF_DEF_VAR(fid,var, NF_FLOAT, ndim, dims, varid)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'axis', LEN_TRIM(axx),axx)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'units', LEN_TRIM(unit), unit) 
    iret = NF_PUT_ATT_TEXT(fid, varid, 'long_name', LEN_TRIM(titre), titre)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'associate', LEN_TRIM(assoc), assoc)
    iret = NF_PUT_ATT_REAL(fid, varid, 'missing_value', NF_FLOAT, 1, undef_sechiba)
    qairid = varid
    !
    var = 'Wind_N'
    unit = 'm/s'
    titre = 'Near surface northward wind component'
    iret = NF_DEF_VAR(fid,var, NF_FLOAT, ndim, dims, varid)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'axis', LEN_TRIM(axx),axx)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'units', LEN_TRIM(unit), unit) 
    iret = NF_PUT_ATT_TEXT(fid, varid, 'long_name', LEN_TRIM(titre), titre)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'associate', LEN_TRIM(assoc), assoc)
    iret = NF_PUT_ATT_REAL(fid, varid, 'missing_value', NF_FLOAT, 1, undef_sechiba)
    uid = varid
    !
    var = 'Wind_E'
    unit = 'm/s'
    titre = 'Near surface eastward wind component'
    iret = NF_DEF_VAR(fid,var, NF_FLOAT, ndim, dims, varid)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'axis', LEN_TRIM(axx),axx)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'units', LEN_TRIM(unit), unit) 
    iret = NF_PUT_ATT_TEXT(fid, varid, 'long_name', LEN_TRIM(titre), titre)
    iret = NF_PUT_ATT_TEXT(fid, varid, 'associate', LEN_TRIM(assoc), assoc)
    iret = NF_PUT_ATT_REAL(fid, varid, 'missing_value', NF_FLOAT, 1, undef_sechiba)
    vid = varid
    !
    ! Time fixed variable
    !
    ndim=2
    dims(1) = nlonid1
    dims(2) = nlatid1
    !
    var = 'contfrac'
    unit = '-'
    titre = 'Fraction of continent'
    assoc = 'nav_lat nav_lon'
    axx='YX'
    iret = NF_DEF_VAR(fid,var, NF_FLOAT, ndim, dims, contid)
    iret = NF_PUT_ATT_TEXT(fid, contid, 'axis', LEN_TRIM(axx),axx)
    iret = NF_PUT_ATT_TEXT(fid, contid, 'units', LEN_TRIM(unit), unit) 
    iret = NF_PUT_ATT_TEXT(fid, contid, 'long_name', LEN_TRIM(titre), titre)
    iret = NF_PUT_ATT_TEXT(fid, contid, 'associate', LEN_TRIM(assoc), assoc)
    iret = NF_PUT_ATT_REAL(fid, contid, 'missing_value', NF_FLOAT, 1, undef_sechiba)
    !
    !  Global attributes
    !
    CALL DATE_AND_TIME(today, att)
    stamp = "Forcing generated by intersurf in a previous run "//today(1:LEN_TRIM(today))//" at "//att(1:LEN_TRIM(att))
    iret = NF_PUT_ATT_TEXT(fid, NF_GLOBAL, 'Conventions', 7, "GDT 1.2")
    iret = NF_PUT_ATT_TEXT(fid, NF_GLOBAL, 'file_name', LEN_TRIM(watchout_file), watchout_file)
    iret = NF_PUT_ATT_TEXT(fid, NF_GLOBAL, 'production', LEN_TRIM(stamp),stamp)
    !
    iret = NF_ENDDEF(fid)
    !
    !    Write coordinates
    !
    iret = NF_PUT_VAR_REAL(fid, nlonid, lon)
    !
    iret = NF_PUT_VAR_REAL(fid, nlatid, lat)
    !
    iret = NF_PUT_VAR_REAL(fid, nlevid, zlev)
    !
    iret = NF_PUT_VAR_INT(fid, nlandid, kindex)
    !
    IF ( .NOT. ALLOCATED(tmpdata)) THEN
       ALLOCATE(tmpdata(iim,jjm))
    ENDIF
    !
    tmpdata(:,:) = undef_sechiba
    DO ig=1,igmax

       j = ((kindex(ig)-1)/iim) + 1
       i = (kindex(ig) - (j-1)*iim)

       tmpdata(i,j) = contfrac(ig)

    ENDDO
    iret = NF_PUT_VAR_REAL(fid, contid, tmpdata)
    !
    DEALLOCATE(tmpdata)
    !
    watchfid = fid
    !
  END SUBROUTINE intersurf_initwatch
  !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !
  SUBROUTINE intersurf_wrtwatch(igmax, itau, dt, soldown, rain, snow, lwdown, psurf, temp, qair, u, v)
    !
    ! 
    IMPLICIT NONE
    !
    ! This subroutine will write to the file the current fields which force the
    ! land-surface scheme. It will be in exactly the same format as the other forcing
    ! files, i.e. ALMA convention !
    !
    !
    ! INPUT
    !
    INTEGER(long) :: igmax, itau
    REAL(stnd) :: soldown(igmax), rain(igmax), snow(igmax), lwdown(igmax), psurf(igmax)
    REAL(stnd) :: temp(igmax), qair(igmax), u(igmax), v(igmax)
    REAL(stnd) :: dt
    !
    ! LOCAL
    !
    INTEGER(long) :: iret
    INTEGER(long) :: corner(3), edges(3)
    REAL(stnd) :: timestp
    LOGICAL    :: check=.FALSE.
    REAL(stnd),ALLOCATABLE :: tmpdata(:)
    !
    !
    INCLUDE "netcdf.inc"
    !
    !
    corner(1) = itau - watchoffset
    edges(1) = 1
    timestp = float(itau)
    IF ( check ) THEN
       WRITE(*,*) 'intersurf_wrtwatch corners, edges : ', corner(1), edges(1)
    ENDIF
    !
    iret = NF_PUT_VARA_REAL(watchfid, timestp_id, corner, edges, timestp)
    iret = NF_PUT_VARA_REAL(watchfid, time_id, corner, edges, timestp*dt)
    !
    corner(1) = 1
    edges(1) = igmax
    corner(2) = itau - watchoffset
    edges(2) = 1
    !
    IF ( .NOT. ALLOCATED(tmpdata)) THEN
       ALLOCATE(tmpdata(igmax))
    ENDIF
    !
    ! 2D
    IF ( check ) THEN
       WRITE(*,*) '--',itau, ' SOLDOWN : ', MINVAL(soldown), MAXVAL(soldown)
    ENDIF
    iret = NF_PUT_VARA_REAL(watchfid, soldownid, corner, edges, soldown)
    !
    ! Bring back to kg/m^2/s
    !
    tmpdata = rain/dt
    IF ( check ) THEN
       WRITE(*,*) '--',itau, ' RAIN : ', MINVAL(tmpdata), MAXVAL(tmpdata)
    ENDIF
    iret = NF_PUT_VARA_REAL(watchfid, rainfid, corner, edges, tmpdata)
    tmpdata = snow/dt
    iret = NF_PUT_VARA_REAL(watchfid, snowfid, corner, edges, tmpdata)
    !
    iret = NF_PUT_VARA_REAL(watchfid, lwradid, corner, edges, lwdown)
    !
    !  Bring back to Pa
    !
    tmpdata = psurf*100.
    iret = NF_PUT_VARA_REAL(watchfid, psolid, corner, edges, tmpdata)
    ! 3D
    corner(2) = 1
    edges(2) = 1
    corner(3) = itau - watchoffset
    edges(3) = 1
    !
    iret = NF_PUT_VARA_REAL(watchfid, tairid, corner, edges, temp)

    iret = NF_PUT_VARA_REAL(watchfid, qairid, corner, edges, qair)

    iret = NF_PUT_VARA_REAL(watchfid, uid, corner, edges, u)

    iret = NF_PUT_VARA_REAL(watchfid, vid, corner, edges, v)
    !
    DEALLOCATE(tmpdata)
    !
  END SUBROUTINE intersurf_wrtwatch
  !
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  !
  SUBROUTINE intersurf_clowatch()
    !
    !  Close the watch files
    !
    IMPLICIT NONE
    !
    ! LOCAL
    !
    INTEGER(long) :: iret
    LOGICAL       :: check = .FALSE.
    !
    !
    INCLUDE "netcdf.inc"
    !
    IF ( check )  THEN
       WRITE(*,*) 'intersurf_clowatch : closing file : ', watchfid
    ENDIF
    iret = NF_CLOSE(watchfid)
    !
  END SUBROUTINE intersurf_clowatch

  SUBROUTINE init_intersurf(nbp_lon,nbp_lat,knon,ktindex,offset,orch_omp_size,orch_omp_rank,orch_comm)

      integer nbp_lon,nbp_lat,knon,ktindex(:),offset,orch_omp_size,orch_omp_rank,orch_comm
      print*,'On ne fait rien dans cette routine avec orchidee beton'
      return
  END SUBROUTINE init_intersurf

  !
END MODULE intersurf
